# cython: embedsignature=True

cdef class CyCoinModel:
    '''

    >>> import numpy as np
    >>> from cylp.cy import CyCoinModel, CyClpSimplex
    >>> m = CyCoinModel()
    >>>
    >>> m.addVariable(3, np.array(
    ...                 [0, 1, 2], np.int32),
    ...                 np.array([1., 1., 1.], np.double), 0, 10, 5)
    >>>
    >>> m.addVariable(2, np.array(
    ...                 [1,2], np.int32),
    ...                 np.array([5, 2.], np.double), 0, 10, 2)
    >>>
    >>> # Add bounds for the three constraints (we have two variables)
    >>> m.setConstraintLower(0, 2.3)
    >>> m.setConstraintLower(1, 4.5)
    >>> m.setConstraintLower(0, 1.5)
    >>>
    >>> # Add a 4th constraint
    >>> m.addConstraint(2,
    ...                     np.array([0, 1], np.int32),
    ...                     np.array([1., 1.], np.double), 2, 7)
    >>>
    >>> s = CyClpSimplex()
    >>> # Load the problem from the CyCoinModel
    >>> s.loadProblemFromCyCoinModel(m)
    0
    >>> s.primal()
    'optimal'
    >>> abs(s.objectiveValue - 8.7) < 10 ** -7
    True

    '''

    def __cinit__(self):
        self.CppSelf = new CppCoinModel()

    def __dealloc__(self):
        del self.CppSelf

#   cdef void CLP_addColumn(self, int numberInColumn,
#                           int * rows,
#                           double * elements,
#                           double columnLower,
#                           double columnUpper,
#                           double objectiveValue,
#                           char * name,
#                           int isInteger):
#       self.CppSelf.addColumn(numberInColumn, rows, elements,
#                              columnLower,columnUpper,
#                              objectiveValue,name, isInteger)

    cdef void CLP_addColumn(self, int numberInColumn,
                            int * rows,
                            double * elements,
                            double columnLower,
                            double columnUpper,
                            double objectiveValue):
        self.CppSelf.addColumn(numberInColumn, rows, elements,
                               columnLower, columnUpper, objectiveValue)

#   def addVariable(self, numberInColumn,
#                       np.ndarray[np.int32_t, ndim=1] rows,
#                       np.ndarray[np.double_t, ndim=1] elements,
#                       columnLower,
#                       columnUpper,
#                       objective,
#                       name,
#                       isInteger):
#        TODO: This makes adding a column real slower,
#         but it is better than a COIN EXCEPTION!
#        for r in rows:
#           if r >= self.getNumRows():
#               raise Exception('CyClpSimplex.pyx:addColumn: Row number ' \
#                                '%d should be less than row size %d' %
#                                (r, self.getNumRows()))
#       self.CLP_addColumn(numberInColumn, <int*>rows.data,
#                            <double*> elements.data, columnLower, columnUpper,
#                            objective, name, isInteger)

    def addVariable(self, numberInColumn,
                        np.ndarray[np.int32_t, ndim=1] rows,
                        np.ndarray[np.double_t, ndim=1] elements,
                        columnLower,
                        columnUpper,
                        objective):
        self.CLP_addColumn(numberInColumn, <int*>rows.data,
                          <double*> elements.data, columnLower,
                          columnUpper, objective)

#   cdef void CLP_addRow(self, int numberInRow, int * columns,
#                       double * elements, double rowLower,
#                       double rowUpper, char * name):
#       self.CppSelf.addRow(numberInRow, columns, elements, rowLower,
#                            rowUpper, name)

    cdef void CLP_addRow(self, int numberInRow, int * columns,
                        double * elements, double rowLower,
                        double rowUpper):
        self.CppSelf.addRow(numberInRow, columns, elements, rowLower, rowUpper)

#   def addConstraint(self, numberInRow,
#                   np.ndarray[np.int32_t, ndim=1] columns,
#                   np.ndarray[np.double_t, ndim=1] elements,
#                   rowLower,
#                   rowUpper,
#                   name = ''):
#       self.CLP_addRow(numberInRow, <int*>columns.data,
#                        <double*>elements.data, rowLower, rowUpper, name)

    def addConstraint(self, numberInRow,
                    np.ndarray[np.int32_t, ndim=1] columns,
                    np.ndarray[np.double_t, ndim=1] elements,
                    rowLower,
                    rowUpper):
        self.CLP_addRow(numberInRow, <int*>columns.data,
                        <double*>elements.data, rowLower, rowUpper)

    def setVariableLower(self, ind, val):
        self.CppSelf.setColumnLower(ind, val)

    def setVariableUpper(self, ind, val):
        self.CppSelf.setColumnUpper(ind, val)

    def setConstraintLower(self, ind, val):
        self.CppSelf.setRowLower(ind, val)

    def setConstraintUpper(self, ind, val):
        self.CppSelf.setRowUpper(ind, val)

    def setObjective(self, varInd, val):
        self.CppSelf.setObjective(varInd, val)

    def getNumVariables(self):
        return self.CppSelf.numberColumns()

    def getNumConstraints(self):
        return self.CppSelf.numberRows()



# cython: embedsignature=True


import numpy as np
cimport numpy as np

# varStatus defined to mimic 'enum ClpSimplex::Status'.
# But is too slow obviously
varStatus = ['isFree', 'basic', 'atUpperBound',
             'atLowerBound', 'superBasic', 'isFixed']

cdef class CyDantzigPivot(CyClpPrimalColumnPivotBase):
    def __init__(self, cyModel):
        self.cyModel = cyModel
        CyClpPrimalColumnPivotBase.__init__(self)

    cdef pivotColumn(self, CppCoinIndexedVector* cppupdates, CppCoinIndexedVector* cppspareRow1,
                    CppCoinIndexedVector* cppspareRow2, CppCoinIndexedVector* cppspareCol1,
                    CppCoinIndexedVector* cppspareCol2):
        updates = CyCoinIndexedVector()
        updates.setCppSelf(cppupdates)
        spareRow1 = CyCoinIndexedVector()
        spareRow1.setCppSelf(cppspareRow1)
        spareRow2 = CyCoinIndexedVector()
        spareRow2.setCppSelf(cppspareRow2)
        spareCol1 = CyCoinIndexedVector()
        spareCol1.setCppSelf(cppspareCol1)
        spareCol2 = CyCoinIndexedVector()
        spareCol2.setCppSelf(cppspareCol2)

        #self.DantzigDualUpdate(v1, v2, v3, v4, v5)
        s = self.cyModel

        # Update the reduced costs, for both the original and the slack variables
        if updates.nElements:
            s.updateColumnTranspose(spareRow2, updates)
            s.transposeTimes(-1, updates, spareCol2, spareCol1)
            s.reducedCosts[s.nVariables:][updates.indices] -= updates.elements[:updates.nElements]
            s.reducedCosts[:s.nVariables][spareCol1.indices] -= spareCol1.elements[:spareCol1.nElements]
        updates.clear()
        spareCol1.clear()

        rc = s.reducedCosts

        cdef double tol = s.dualTolerance

        #incides of vars not fixed and not flagged
        #indicesToConsider = np.where((status & 7 != 1) & (status & 7 != 5) &
        #        (status & 64 == 0) & (((rc > tol) & (status & 7 == 2)) |
        #            ((rc < -tol) & (status & 7 == 3))) )[0]

        cdef np.ndarray indicesToConsider = \
                                np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree))[0]

        #freeVarInds = np.where(s.varIsFree)
        #rc[freeVarInds] *= 10

        cdef np.ndarray rc2 = abs(rc[indicesToConsider])

        cdef int checkFree = 0
        cdef int ind
        #rc2[np.where((status & 7 == 4) | (status & 7 == 0))] *= 10
        if rc2.shape[0] > 0:
            if checkFree:
                w = np.where(s.varIsFree)[0]
                if w.shape[0] > 0:
                    ind = s.argWeightedMax(rc2, indicesToConsider, 1, w)
                else:
                    ind = np.argmax(rc2)
            else:
                    ind = np.argmax(rc2)
            return  indicesToConsider[ind]
        return -1

    cdef CyClpPrimalColumnPivot* clone(self, bint copyData):
        cdef CyClpPrimalColumnPivot* ret =  \
                <CyClpPrimalColumnPivot*> new CppClpPrimalColumnPivotBase(
                                <cpy_ref.PyObject*>self,
                                <runPivotColumn_t>RunPivotColumn,
                                <runClone_t>RunClone,
                                <runSaveWeights_t>RunSaveWeights)
        return ret

    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex * model, int mode):
        self.CppSelf.setModel(model)



# cython: embedsignature=True

cdef class CyCoinIndexedVector:
    '''
    ``CyCoinIndexedVector`` interfaces ``CoinIndexedVector``.

    **Usage**

    >>> from cylp.cy import CyCoinIndexedVector
    >>> vector = CyCoinIndexedVector()
    >>> vector.reserve(5)
    >>> vector[3] = 4
    >>> vector[3]
    4.0
    >>> vector[[1, 3, 4]] = 2.1
    >>> [vector[i] for i in [1, 3, 4]] == 3 * [2.1]
    True
    >>> vector[[0, 2]] = [-1, -2]
    >>> [vector[i] for i in [0, 2]] == [-1, -2]
    True
    '''

    def __cinit__(self):
        self.CppSelf = new_CppCoinIndexedVector()

    #def __dealloc__(self):
    #    del self.CppSelf

    cdef setCppSelf(self, CppCoinIndexedVector* s):
        del self.CppSelf
        self.CppSelf = s

    def Print(self):
        self.CppSelf.Print()

    cpdef reserve(self, n):
        self.CppSelf.reserve(n)

    def __getitem__(self, n):
        return self.CppSelf.getItem(n)

    def __setitem__(self, key, value):
        self.assign(key, value)
        #self.CppSelf.insert(key, value)
        #self.CppSelf.setItem(key, value)

    #def __setslice__(self, rg, val):
        #print(rg, val)

    def clear(self):
        self.CppSelf.clear()

    def empty(self):
        self.CppSelf.empty()

    cpdef assign(self, ind, other):
        self.CppSelf.assign(<PyObject*>ind, <PyObject*>other)

    property indices:
        def __get__(self):
            return <object>self.CppSelf.getIndicesNPArray()

    property elements:
        def __get__(self):
            return <object>self.CppSelf.getDenseVectorNPArray()

    property nElements:
        def __get__(self):
            return self.CppSelf.getNumElements()

        def __set__(self, value):
            self.CppSelf.setNumElements(value)

    property isInPackedMode:
        def __get__(self):
            return self.CppSelf.packedMode()



import numpy as np
from math import atan2
from cylp.py import Constants
from operator import mul

def sign(x):
    if x > 0 or (x == 0 and atan2(x, -1.) > 0.):
        return 1
    else:
        return -1

def get_cs(w1, w2):
    omega = float(sign(w1) * (w1**2 + w2**2)**0.5)
    c = w1 / omega
    s = w2 / omega
    return c, s

def givens(n, i, j, w1, w2):
    g = np.identity(n, float)
    if abs(w1) < Constants.EPSILON and abs(w2) < Constants.EPSILON:
        return g
    c, s = get_cs(w1, w2)
    g[i,i], g[j,j], g[i,j], g[j,i] = c, -c, s, s
    return g

def applyGivens(vec):
    'Applies dim-1 givens matrices so that vec contains only one non-zero element'
    v = vec.copy()
    dim = v.shape[0]
    Q_bar = np.matrix(np.identity(dim, float))
    Qlist = []
    for i in range(dim - 1):
        if vec[dim-i-1] != 0:
            Q = givens(dim, dim-i-2, dim-i-1, v[dim-i-2,0], v[dim-i-1,0])
            Qlist.append(Q)
            v = Q * v
            Q_bar = Q * Q_bar
    return Qlist, Q_bar, v[0,0]

def UH2UT(mat):
    m = mat.copy()
    nrows, ncols = mat.shape
    Q_bar = np.matrix(np.identity(nrows, float))
    for i in range(ncols):
        Q = givens(nrows, i, i+1, m[i,i], m[i+1,i])
        m = Q * m
        Q_bar = Q * Q_bar
    #we need to remove the last line because it's all zero (and we need a square matrix)
    return Q_bar, m[:nrows-1, :]


## This part is for defining the decorators 'precondtion' and 'postcondition' and 'conditions'

__all__ = ['precondition', 'postcondition', 'conditions']

DEFAULT_ON = True

def precondition(precondition, use_conditions=DEFAULT_ON):
    return conditions(precondition, None, use_conditions)

def postcondition(postcondition, use_conditions=DEFAULT_ON):
    return conditions(None, postcondition, use_conditions)

class conditions(object):
    __slots__ = ('__precondition', '__postcondition')

    def __init__(self, pre, post, use_conditions=DEFAULT_ON):
        if not use_conditions:
            pre, post = None, None

        self.__precondition = pre
        self.__postcondition = post

    def __call__(self, function):
        # combine recursive wrappers (@precondition + @postcondition == @conditions)
        pres = set((self.__precondition,))
        posts = set((self.__postcondition,))

        # unwrap function, collect distinct pre-/post conditions
        while type(function) is FunctionWrapper:
            pres.add(function._pre)
            posts.add(function._post)
            function = function._func

        # filter out None conditions and build pairs of pre- and postconditions
        conditions = map(None, filter(None, pres), filter(None, posts))

        # add a wrapper for each pair (note that 'conditions' may be empty)
        for pre, post in conditions:
            function = FunctionWrapper(pre, post, function)

        return function

class FunctionWrapper(object):
    def __init__(self, precondition, postcondition, function):
        self._pre = precondition
        self._post = postcondition
        self._func = function

    def __call__(self, *args, **kwargs):
        precondition = self._pre
        postcondition = self._post

        if precondition:
            precondition(*args, **kwargs)
        result = self._func(*args, **kwargs)
        if postcondition:
            postcondition(result, *args, **kwargs)
        return result


class Ind:
    def __init__(self, key, dim):
        '''
        Create an instance of Ind using *key* that can be
        an integer, a slice, a list, or a numpy array.
        '''
        if isinstance(key, slice):
            sl = key
            if sl.stop and (sl.start > dim or sl.start >= sl.stop):
                raise Exception('Indexing problem: %s, dim=%d:' % (str(sl), dim))

            if  not sl.stop or sl.stop > dim:
                stop = dim
            else:
                stop = sl.stop
            if not sl.start:
                start = 0
            else:
                start = sl.start
            self.indices = range(start, stop)
            self.dim = dim
        elif isinstance(key, int):
            if key >= dim:
                raise Exception('Index (%d) out of range (%d)' % (key, dim))
            self.indices = [key]
            self.dim = dim
        elif isinstance(key, (list, np.ndarray)):
            self.indices = key
            self.dim = dim
            # Here to avoid checking all the inds, I suppose that normally
            # the list of indices is sorted. So just check the last number
            if key[-1] >= dim:
                raise Exception('Index (%d) out of range (%d)' % (key[-1], dim))
        else:
            raise Exception('Error indexing with unrecognized type %s' % key.__class__)


    def __repr__(self):
        return '(%d, %d / %d)' % (self.start, self.stop, self.dim)

def getIndS(inds):
    n = len(inds)
    if n == 1:
        return inds[0].start
    prod = inds[0].stop
    for i in range(1, n):
        prod *= inds[i].dim
    return prod + getIndS(inds[1:])

def getMultiDimMatrixIndex(inds, res=[]):
    n = len(inds)
    r = inds[0].indices
    #r = range(inds[0].start, inds[0].stop)
    if n == 1:
        return r
    l = []
    for i in r:
        prod = i
        for k in range(1, n):
            prod *= inds[k].dim
        rest = getMultiDimMatrixIndex(inds[1:], res)
        l += res + [prod + rs for rs in rest]
    return l

def getTupleIndex(ind, dims):
    if isinstance(dims, int):
        return [ind] if ind < dims else -1
    n = len(dims)
    if ind > reduce(mul, dims):
        return -1
    if n == 1:
        return [ind]
    #return getTupleIndex(ind / dims[-1], dims[:-1]) + [ind % dims[-1]]
    ret = []
    for i in range(n):
        d = dims[n - i - 1]
        ret.insert(0, ind % d)
        ind /= d
        #return getTupleIndex(ind / dims[-1], dims[:-1]) + [ind % dims[-1]]
    return ret

if __name__ == '__main__':
    i1 = Ind(slice(1, 4), 5)
    i2 = Ind(slice(2, 4), 6)
    #i3 = Ind(slice(2, 5), 7)
    i3 = Ind(np.array([1, 4, 6]), 7)


    inds = getMultiDimMatrixIndex([i1, i2, i3])

    for i in inds:
        print(getTupleIndex(i, (5, 6, 7)), i)

    print(getTupleIndex(8, 8))



'''
This module interface COIN-OR's ``CoinMpsIO``. When you call
:func:`cylp.cy.CyClpSimplex.readMps` then ``CoinMpsIO``'s ``readMps`` is
called.  The main reason why cylp interfaces this class is to be able to read
an ``mps`` file without creating a Simplex object. This way it is possible to
read a QP using CoinMpsIO and work on the elements of the problem, e.g. the
Hessian,...
'''
# cython: embedsignature=True

import numpy as np
from scipy import sparse
from scipy.sparse import identity, dia_matrix
from cylp.py.utils.sparseUtil import csc_matrixPlus, csr_matrixPlus

cdef class CyCoinMpsIO:
    def __cinit__(self):
        self.CppSelf = new CppICoinMpsIO()
        self.Hessian = 0

    def readMps(self, filename):
        '''
        Read an mps file. Check if the file is a QP symmetrisize its Hessian
        and store it.

        >>> import numpy as np
        >>> from cylp.cy import CyCoinMpsIO
        >>> from cylp.cy.CyCoinMpsIO import getQpsExample
        >>> problem = CyCoinMpsIO()
        >>> problem.readMps(getQpsExample())
        0
        >>> problem.nVariables
        5
        >>> problem.nConstraints
        5
        >>> signs = problem.constraintSigns
        >>> [chr(i) for i in signs] == problem.nConstraints * ['G']
        True
        >>> c = problem.matrixByRow
        >>> (abs(c.elements -
        ...     np.array([-1., -1., -1., -1., -1.,  10.,  10.,  -3.,
        ...                5., 4.,  -8., 1., -2., -5., 3., 8., -1., 2.,
        ...                5., -3.,  -4.,  -2., 3., -5., 1.])) <
        ...                            10 ** -8).all()
        True
        >>> (c.indices ==
        ...       np.array([0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 0, 1,
        ...                 2, 3, 4, 0, 1, 2, 3, 4], dtype=np.int32)).all()
        True
        >>> (c.vectorStarts ==
        ...        np.array([0, 5, 10, 15, 20, 25], dtype=np.int32)).all()
        True
        >>> (problem.rightHandSide ==
        ...        np.array([-5., 20., -40., 11., -30.])).all()
        True
        >>> H = problem.Hessian.todense()
        >>> (abs(H -
        ... np.matrix([[20394., -24908., -2026., 3896., 658.],
        ...            [-24908., 41818., -3466., -9828., -372.],
        ...            [-2026., -3466., 3510., 2178., -348.],
        ...            [3896., -9828., 2178., 3030., -44.],
        ...            [658., -372., -348., -44., 54.]])) <
        ...                            10 ** -8).all()
        True

        '''
        if isinstance(filename, str):
            # Cast strings/unicode to bytes
            filename = filename.encode('utf-8')

        ret = self.CppSelf.readMps(filename)
        if ret == 0 and self.CppSelf.IreadQuadraticMps(NULL, 0) == 0:
            start = self.QPColumnStarts
            col = self.QPColumns
            el = self.QPElements

            n = self.nVariables
            m = self.nConstraints

            Hessian = csr_matrixPlus((el, col, start), shape=(n, n))
            # Fill the other half of the symmetric Hessian
            Hessian = Hessian + Hessian.T - dia_matrix((Hessian.diagonal(),[0]), shape=(n, n))
            self.Hessian = Hessian

        return ret

    def readQuadraticMps(self, filename, checkSymmetry):
        return self.CppSelf.IreadQuadraticMps(NULL, checkSymmetry)

    property Hessian:
        def __get__(self):
            return self.Hessian

        def __set__(self, h):
            self.Hessian = h

    property variableLower:
        def __get__(self):
            return <object>self.CppSelf.np_getColLower()

    property variableUpper:
        def __get__(self):
            return <object>self.CppSelf.np_getColUpper()

    property constraintSigns:
        def __get__(self):
            return <object>self.CppSelf.np_getRowSense()

    property rightHandSide:
        def __get__(self):
            return <object>self.CppSelf.np_getRightHandSide()

    property constraintRange:
        def __get__(self):
            return <object>self.CppSelf.np_getRowRange()

    property constraintLower:
        def __get__(self):
            return <object>self.CppSelf.np_getRowLower()

    property constraintUpper:
        def __get__(self):
            return <object>self.CppSelf.np_getRowUpper()

    property objCoefficients:
        def __get__(self):
            return <object>self.CppSelf.np_getObjCoefficients()

    property integerColumns:
        def __get__(self):
            return <object>self.CppSelf.np_integerColumns()

    property QPColumnStarts:
        def __get__(self):
            return <object>self.CppSelf.getQPColumnStarts()

    property QPColumns:
        def __get__(self):
            return <object>self.CppSelf.getQPColumns()

    property QPElements:
        def __get__(self):
            return <object>self.CppSelf.getQPElements()

    property matrixByRow:
        def __get__(self):
            cdef CppCoinPackedMatrix* m = self.CppSelf.IgetMatrixByRow()
            cym = CyCoinPackedMatrix()
            cym.CppSelf = m
            return cym

    property matrixByCol:
        def __get__(self):
            cdef CppCoinPackedMatrix* m = self.CppSelf.IgetMatrixByCol()
            cym = CyCoinPackedMatrix()
            cym.CppSelf = m
            return cym

    property objectiveOffset:
        def __get__(self):
            return self.CppSelf.getObjectiveOffset()

    property nVariables:
        def __get__(self):
            return self.CppSelf.getNumCols()

    property nConstraints:
        def __get__(self):
            return self.CppSelf.getNumRows()


def getQpsExample():
    '''
    Return full path to a QPS example file for doctests
    '''
    import os
    import inspect
    curpath = os.path.dirname(inspect.getfile(inspect.currentframe()))
    return os.path.join(curpath, '../input/hs268.qps')


#   cdef int CLP_readQuadraticMps(self, char* filename,
#               int * columnStart, int * column2, double * elements,
#               int checkSymmetry):
#       return self.CppSelf.readQuadraticMps(filename, columnStart,
#                                        column2, elements, checkSymmetry)

#   def readQuadraticMps(self,
#                           filename,
#                           np.ndarray[np.int32_t,ndim=1] columnStart,
#                           np.ndarray[np.int32_t,ndim=1] column2,
#                           np.ndarray[np.double_t,ndim=1] elements,
#                           checkSymmetry):
#       return self.CLP_readQuadraticMps(filename,
#                                           <int*>columnStart.data,
#                                           <int*>column2.data,
#                                           <double*>elements.data,
#                                           checkSymmetry)



# cython: embedsignature=True

import numpy as np
cimport numpy as np


cdef class CyCoinPackedMatrix:
    '''
    ``CyCoinPackedMatrix`` interfaces ``CoinPackedMatrix``

    **Usage Example**

    >>> import numpy as np
    >>> from cylp.cy import CyCoinPackedMatrix
    >>> rows = np.array([1, 3, 4], np.int32)
    >>> cols = np.array([0, 2, 1], np.int32)
    >>> elements = np.array([1.5, -1, 2])
    >>> # colOrdered is True if we store the matrix by column (csc)
    >>> m = CyCoinPackedMatrix(colOrdered=True, rowIndices=rows,
    ...                           colIndices=cols, elements=elements)
    >>> m.majorDim
    3
    >>> m.minorDim
    5
    '''
    def __cinit__(self, colOrdered=True,
                        np.ndarray[np.int32_t, ndim=1] rowIndices=None,
                        np.ndarray[np.int32_t, ndim=1] colIndices=None,
                        np.ndarray[np.double_t, ndim=1] elements=None):
        if rowIndices is None:
            self.CppSelf = new_CppCoinPackedMatrix()
        else:
            self.CppSelf = new CppCoinPackedMatrix(colOrdered,
                                    <int*>rowIndices.data,
                                    <int*> colIndices.data,
                                    <double*> elements.data,
                                    len(rowIndices))

    property indices:
        def __get__(self):
            return <object>self.CppSelf.np_getIndices()

    property elements:
        def __get__(self):
            return <object>self.CppSelf.np_getElements()

    property vectorStarts:
        def __get__(self):
            return <object>self.CppSelf.np_getVectorStarts()

    #cpdef reserve(self, n):
    #   self.CppSelf.reserve(n)

    property nElements:
        def __get__(self):
            return self.CppSelf.getNumElements()

    property majorDim:
        def __get__(self):
            return self.CppSelf.getMajorDim()

    property minorDim:
        def __get__(self):
            return self.CppSelf.getMinorDim()

    property isColOrdered:
        def __get__(self):
            return self.CppSelf.isColOrdered()

    def reserve(self,  newMaxMajorDim,  newMaxSize,  create=0):
        self.CppSelf.reserve(newMaxMajorDim, newMaxSize, create)

    def appendRow(self, np.ndarray[np.int32_t, ndim=1] vecInd=None,
                  np.ndarray[np.double_t, ndim=1] elements=None):

        cdef int* i
        cdef double* d
        if vecInd:
            self.CppSelf.appendRow(len(elements), <int*>vecInd.data,
                                            <double*>elements.data)
        else:
            self.CppSelf.appendRow(0, i, d)

    def appendCol(self, np.ndarray[np.int32_t, ndim=1] vecInd=None,
                  np.ndarray[np.double_t, ndim=1] elements=None):

        cdef int* i
        cdef double* d
        if vecInd:
            self.CppSelf.appendCol(len(elements), <int*>vecInd.data,
                                            <double*>elements.data)
        else:
            self.CppSelf.appendCol(0, i, d)

    def dumpMatrix(self, char* s):
        #if s:
        self.CppSelf.dumpMatrix(s)
        #else:
        #    self.CppSelf.dumpMatrix(s)

    def hasGaps(self):
        return self.CppSelf.hasGaps()

    def removeGaps(self, removeValue=-1.0):
        self.CppSelf.removeGaps(removeValue)

    #def __getitem__(self, n):
    #   return self.CppSelf.getItem(n)

    #def __setitem__(self, key, value):
    #   self.CppSelf.setItem(key, value)



# cython: c_string_type=str, c_string_encoding=ascii
# cython: profile=True
# cython: embedsignature=True

import inspect
import os.path
from itertools import product
import numpy as np
cimport numpy as np
from scipy import sparse
cimport cpython.ref as cpy_ref
from cylp.cy.CyWolfePivot cimport CyWolfePivot
from cylp.cy.CyPEPivot cimport CyPEPivot
from cylp.cy.CyPivotPythonBase cimport CyPivotPythonBase
from cylp.cy.CyDualPivotPythonBase cimport CyDualPivotPythonBase
from cylp.cy cimport CyCoinModel
from cylp.py.utils.sparseUtil import sparseConcat, csc_matrixPlus
from cylp.py.modeling.CyLPModel import CyLPVar, CyLPArray, CyLPSolution
from cylp.py.pivots.PivotPythonBase import PivotPythonBase
from cylp.py.pivots.DualPivotPythonBase import DualPivotPythonBase
from cylp.py.modeling.CyLPModel import CyLPModel
from cylp.cy cimport CyCoinMpsIO

# Initialize numpy
np.import_array()

problemStatus = ['optimal', 'primal infeasible', 'dual infeasible',
                'stopped on iterations or time',
                'stopped due to errors',
                'stopped by event handler (virtual int ' \
                                    'ClpEventHandler::event())']

CLP_variableStatusEnum = [0x00, 0x01, 0x02, 0x03, 0x04, 0x05]
StatusToInt = {'free' : 0, 'basic' : 1, 'atUpperBound' : 2,
                             'atLowerBound' : 3, 'superBasic' : 4,'fixed' : 5}
IntToStatus = ['free', 'basic', 'atUpperBound',
                             'atLowerBound', 'superBasic','fixed']

startFinishOptionsDic = {'x' : 1,  #do not delete work areas
                      'f' : 2, #use old factorization
                      's' : 4} #skip initialization of work areas

cdef class CyClpSimplex:
    '''
    CyClpSimplex is a Cython interface to CLP.
    Not all methods are available but they are being added gradually.

    Its constructor can create an empty object if no argument is provided.
    However, if a :class:`CyLPModel <cylp.py.modeling.CyLPModel>` object is
    given then the resulting ``CyClpSimplex`` object will be build from it.
    For an example of the latter case see
    :mod:`cylp's modeling tool. <cylp.py.modeling.CyLPModel>`

    .. _simple-run:

    **An easy example of how to read and solve an LP**

    >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getMpsExample
    >>> s = CyClpSimplex()
    >>> f = getMpsExample()
    >>> s.readMps(f)
    0
    >>> s.initialSolve()
    'optimal'

    '''

    def __cinit__(self, cyLPModel=None):
        self.CppSelf = new CppIClpSimplex(<cpy_ref.PyObject*>self,
                                <runIsPivotAcceptable_t>RunIsPivotAcceptable,
                                <varSelCriteria_t>RunVarSelCriteria)
        self.vars = []
        #self.cbcModelExists = False
        self.coinModel = CyCoinModel()

        self.cyLPModel = cyLPModel
        if cyLPModel:
            if isinstance(cyLPModel, CyLPModel):
                self.loadFromCyLPModel(cyLPModel)
            else:
                raise TypeError('Expected a CyLPModel as an argument to ' \
                                'cylpSimplex constructor. Got %s' %
                                cyLPModel.__class__)

    def __dealloc__(self):
        del self.CppSelf

    cdef setCppSelf(self,  CppIClpSimplex* s):
        del self.CppSelf
        self.CppSelf = s

    #############################################
    # Properties
    #############################################

    property basicVariables:
        def __get__(self):
            cdef np.ndarray[np.int32_t, ndim=1] bv = np.zeros(self.nConstraints, dtype=np.int32)
            self.CppSelf.getBasics(<int*>bv.data)
            return bv

    property rhs:
        def __get__(self):
            r = np.zeros(self.nConstraints, dtype=np.double)
            self.getRightHandSide(r)
            #Py_INCREF(r)
            return r

    property basisInverse:
        def __get__(self):
            b = np.empty((self.nConstraints, self.nConstraints), dtype=np.double)
            cdef np.ndarray[np.double_t, ndim=1] c = np.zeros(self.nConstraints, dtype=np.double)
            for colInd in xrange(self.nConstraints):
                self.getBInvCol(colInd, c)
                b[:, colInd] = c
            return b

    property tableau:
        def __get__(self):
            nAllVars = self.nVariables + self.nConstraints
            t = np.empty((self.nConstraints, nAllVars), dtype=np.double)
            cdef np.ndarray[np.double_t, ndim=1] c = np.zeros(self.nConstraints, dtype=np.double)
            for colInd in xrange(nAllVars):
                self.getBInvACol(colInd, c)
                t[:, colInd] = c
            return t

    property objective:
        '''
        Set the objective function using this property.
        See the :ref:`modeling example <modeling-usage>`.
        '''
        def __set__(self, obj):
            if self.cyLPModel:
                self.cyLPModel.objective = obj
                o = self.cyLPModel.objective

                if not isinstance(o, (np.ndarray)):
                    o = o.toarray()[0]
                self.setObjectiveArray(o.astype(np.double))
                # This old version doesn't work in some versions of Scipy
                # For csr_matrixPlus, o[0,i] is still a matrix, not a number
                # This does work in some versions of SciPy
                # It would probably be OK if csr_matrixPlus didn't override
                # __get_item__ to always cast the result back to csr_matrixPlus
                # I'm not actually sure why the objective is stored as 
                # csr_matrixPlus anyway... seems to not always be true.
                #
                #if isinstance(o, (sparse.coo_matrix,
                #                                sparse.csc_matrix,
                #                                sparse.csr_matrix,
                #                                sparse.lil_matrix)):
                #    for i in xrange(self.nVariables):
                #        self.setObjectiveCoefficient(i, o[0,i])
                    #if not isinstance(o, sparse.coo_matrix):
                    #    o = o.tocoo()
                    #for i, j, v in zip(o.row, o.col, o.data):
                    #    self.setObjectiveCoefficient(j, v)
                #self.setObjectiveArray(
                #       self.cyLPModel.objective.astype(np.double))
            else:
                raise Exception('To set the objective function of ' \
                                'CyClpSimplex set cylpSimplex.cyLPModel ' \
                                'first.')
        def __get__(self):
            return <object>self.CppSelf.getObjective()

    property objectiveCoefficients:
        '''
        An alternative to self.objective, with a more meaningful name
        in a more general context. Sets and gets a numpy array.
        '''
        def __set__(self, coef):
            self.objective = coef

        def __get__(self):
            return self.objective

    property iteration:
        '''
        Number of iterations.
        '''
        def __get__(self):
            return self.CppSelf.numberIterations()

    property nRows:
        '''
        Number of rows, constraints.
        '''
        def __get__(self):
            return self.CppSelf.getNumRows()

    property nConstraints:
        '''
        Number of constraints, rows.
        '''
        def __get__(self):
            return self.CppSelf.getNumRows()

    property nVariables:
        '''
        Number of variables, columns.
        '''
        def __get__(self):
            return self.CppSelf.getNumCols()

    property nCols:
        '''
        Number of columns, variables.
        '''
        def __get__(self):
            return self.CppSelf.getNumCols()

    property coefMatrix:
        '''
        The coefficient matrix. A scipy sparse matrix.
        '''
        def __get__(self):
            mat = self.matrix
            if mat.hasGaps():
                mat.removeGaps()
            return csc_matrixPlus((mat.elements, mat.indices, mat.vectorStarts),
                             shape=(self.nConstraints, self.nVariables))

        def __set__(self, sparseMatrix):
            try:
                m = sparseMatrix.tocoo()
            except:
                raise Exception('coefMatrix must be a scipy sparse matrix.')
            self.matrix = CyCoinPackedMatrix(True, m.row, m.col, m.data)

    property matrix:
        '''
        The coefficient matrix. A CyCoinPackedMatrix.
        '''
        def __get__(self):
            cdef CppCoinPackedMatrix* cppMat = self.CppSelf.getMatrix()
            mat = CyCoinPackedMatrix()
            mat.CppSelf = cppMat
            return mat

        def __set__(self, cyCoinPackedMatrix):
            self.replaceMatrix(cyCoinPackedMatrix, True)

    property constraints:
        '''
        Constraints.
        '''
        def __get__(self):
            if not self.cyLPModel:
                raise Exception('No CyClpSimplex cyLPModel.')
            else:
                return self.cyLPModel.constraints

    property variableNames:
        '''
        variable names
        '''
        def __get__(self):
            return self.getVariableNames()

    property variables:
        '''
        Variables.
        '''
        def __get__(self):
            if not self.cyLPModel:
                raise Exception('No CyClpSimplex cyLPModel.')
            else:
                return self.cyLPModel.variables

#    def getNumRows(self):
#        '''
#        Return number of constraints
#        '''
#        return self.CppSelf.getNumRows()

#    def getNumCols(self):
#        return self.CppSelf.getNumCols()

    property objectiveValue:
        '''
        The objective value. Readonly.
        '''
        def __get__(self):
            return self.CppSelf.objectiveValue()

    property primalVariableSolution:
        '''
        Solution to the primal variables.

        :rtype: Numpy array
        '''
        def __get__(self):
            #if self.cbcModelExists:
            #    return <object>self.cbcModel.getPrimalVariableSolution()
            ret = <object>self.CppSelf.getPrimalColumnSolution()
            if self.cyLPModel:
                m = self.cyLPModel
                inds = m.inds
                d = {}
                for v in inds.varIndex.keys():
                    d[v] = ret[inds.varIndex[v]]
                    var = m.getVarByName(v)
                    if var.dims:
                        d[v] = CyLPSolution()
                        dimRanges = [range(i) for i in var.dims]
                        for element in product(*dimRanges):
#                            d[v][element] = ret[var.__getitem__(element).indices[0]]
                            d[v][element] = ret[var.fromInd+var[element].indices[0]]
                ret = d
            else:
                names = self.variableNames
                if names:
                    d = CyLPSolution()
                    for i in range(len(names)):
                        d[names[i]] = ret[i]
                    ret = d
            return ret

    property primalVariableSolutionAll:
        '''
        Solution to the primal variables. Including the slacks.

        :rtype: Numpy array
        '''
        def __get__(self):
            #if self.cbcModelExists:
            #    return <object>self.cbcModel.getPrimalVariableSolution()
            return <object>self.CppSelf.getPrimalColumnSolutionAll()

    property solution:
        '''
        Return the current point.

        :rtype: Numpy array
        '''
        def __get__(self):
            #if self.cbcModelExists:
            #    return <object>self.cbcModel.getPrimalVariableSolution()
            return <object>self.CppSelf.getSolutionRegion()

    property cost:
        '''
        Return the cost vector.

        :rtype: Numpy array
        '''
        def __get__(self):
            #if self.cbcModelExists:
            #    return <object>self.cbcModel.getPrimalVariableSolution()
            return <object>self.CppSelf.getCostRegion()

    property dualVariableSolution:
        '''
        Variables' reduced costs

        :rtype: Numpy array
        '''
        def __get__(self):
            ret = <object>self.CppSelf.getDualColumnSolution()
            if self.cyLPModel:
                m = self.cyLPModel
                inds = m.inds
                d = {}
                for v in inds.varIndex.keys():
                    d[v] = ret[inds.varIndex[v]]
                    var = m.getVarByName(v)
                    if var.dims:
                        d[v] = CyLPSolution()
                        dimRanges = [range(i) for i in var.dims]
                        for element in product(*dimRanges):
                            d[v][element] = ret[var.__getitem__(element).indices[0]]
                ret = d
            else:
                names = self.variableNames
                if names:
                    d = CyLPSolution()
                    for i in range(len(names)):
                        d[names[i]] = ret[i]
                    ret = d
            return ret

    property primalConstraintSolution:
        '''
        Slack variables' solution

        :rtype: Numpy array
        '''
        def __get__(self):
            ret = <object>self.CppSelf.getPrimalRowSolution()
            if self.cyLPModel:
                m = self.cyLPModel
                inds = m.inds
                d = {}
                for c in inds.constIndex.keys():
                    d[c] = ret[inds.constIndex[c]]
                ret = d
            else:
                pass
                #names = self.variableNames
                #if names:
                #    d = CyLPSolution()
                #    for i in range(len(names)):
                #        d[names[i]] = ret[i]
                #    ret = d
            return ret


    property dualConstraintSolution:
        '''
        Dual variables' solution

        :rtype: Numpy array
        '''
        def __get__(self):
            ret =  <object>self.CppSelf.getDualRowSolution()
            if self.cyLPModel:
                m = self.cyLPModel
                inds = m.inds
                d = {}
                for c in inds.constIndex.keys():
                    d[c] = ret[inds.constIndex[c]]
                ret = d
            else:
                pass
                #names = self.variableNames
                #if names:
                #    d = CyLPSolution()
                #    for i in range(len(names)):
                #        d[names[i]] = ret[i]
                #    ret = d
            return ret

    property reducedCosts:
        '''
        The reduced costs. A Numpy array.

        :rtype: Numpy array
        '''
        def __get__(self):
            return self.getReducedCosts()

        def __set__(self, np.ndarray[np.double_t, ndim=1] rc):
            self.CppSelf.setReducedCosts(<double*> rc.data)

    cpdef getReducedCosts(self):
        return <object>self.CppSelf.getReducedCosts()

    property objectiveOffset:
        '''
        The constant value in the objective function. A float.
        '''
        def __get__(self):
            return self.CppSelf.objectiveOffset()

        def __set__(self, value):
            self.CppSelf.setObjectiveOffset(value)

    property variablesUpper:
        '''
        Variables upper bounds

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getColUpper()

        def __set__(self, upperArray):
            self.setColumnUpperFirstElements(upperArray)

    property variablesLower:
        '''
        Variables lower bounds

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getColLower()

        def __set__(self, lowerArray):
            self.setColumnLowerFirstElements(lowerArray)

    property constraintsUpper:
        '''
        Constraints upper bounds

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getRowUpper()

        def __set__(self, upperArray):
            self.setRowUpperArray(upperArray)

    property constraintsLower:
        '''
        Constraints lower bounds

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getRowLower()

        def __set__(self, lowerArray):
            self.setRowLowerArray(lowerArray)

    property lower:
        '''
        lower bounds (CLP's lower_)

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getLower()

    property upper:
        '''
        upper bounds (CLP's upper_)

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getUpper()

    property variableScale:
        '''
        Array of variables' scale factors

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getColumnScale()

    property constraintScale:
        '''
        Array of constraints' scale factors

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getRowScale()

    property integerInformation:
        '''
        A binary list of size *nVariables* that specifies whether
        a variable is integer or not. (ClpModel::integerInformation())

        :rtype: Numpy array
        '''
        def __get__(self):
            return <object>self.CppSelf.getIntegerInformation()

    property status:
        '''
        A Numpy array of all the variables' status
        '''
        def __get__(self):
            return self.getStatusArray()

    cpdef getStatusArray(self):
        return <object>self.CppSelf.getStatusArray()

    property freeOrSuperBasicVarInds:
        '''
        The index set of variables that are *free* or *superbasic*.
        '''
        def __get__(self):
            status = self.status
            return np.where((status & 7 == 4) | (status & 7 == 0))[0]

    property notBasicOrFixedOrFlaggedVarInds:
        '''
        The index set of variables that are not *basic* or *fixed*.
        '''
        def __get__(self):
            status = self.status
            return np.where((status & 7 != 1) &
                            (status & 7 != 5) &
                            (status & 64 == 0))[0]

    property varIsFree:
        '''
        The index set of variables that are *free*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 0)

    property varIsBasic:
        '''
        The index set of variables that are *basic*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 1)

    property varIsAtUpperBound:
        '''
        The index set of variables that are at their upper bound.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 2)

    property varIsAtLowerBound:
        '''
        The index set of variables that are at their lower bound.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 3)

    property varIsSuperBasic:
        '''
        The index set of variables that are *superbasic*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 4)

    property varIsFixed:
        '''
        The index set of variables that are *fixed*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 == 5)

    property varIsFlagged:
        '''
        The index set of variables that are *flagged*.
        '''
        def __get__(self):
            status = self.status
            return (status & 64 != 0)

    property varNotFree:
        '''
        The index set of variables that are NOT *free*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 0)

    property varNotBasic:
        '''
        The index set of variables that are NOT *basic*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 1)

    property varNotAtUpperBound:
        '''
        The index set of variables that are NOT at their upper bound.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 2)

    property varNotAtLowerBound:
        '''
        The index set of variables that are NOT at their lower bound.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 3)

    property varNotSuperBasic:
        '''
        The index set of variables that are NOT *superbasic*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 4)

    property varNotFixed:
        '''
        The index set of variables that are NOT *fixed*.
        '''
        def __get__(self):
            status = self.status
            return (status & 7 != 5)

    property varNotFlagged:
        '''
        The index set of variables that are NOT flagged.
        '''
        def __get__(self):
            status = self.status
            return (status & 64 == 0)

    property Hessian:
        def __get__(self):
            return self._Hessian

        def __set__(self, mat):
            m = None
            try:
                m = mat.tocoo()
            except:
                raise Exception('Hessian can be set to a matrix that ' \
                                            'implements *tocoo* method')
            if m:
                coinMat = CyCoinPackedMatrix(True, m.row, m.col, m.data)
                n = self.nVariables
                if coinMat.majorDim < n:
                    for i in xrange(n - coinMat.majorDim):
                        coinMat.appendCol()
                if coinMat.minorDim < n:
                    for i in xrange(n - coinMat.majorDim):
                        coinMat.appendRow()
                self._Hessian = m
            self.loadQuadraticObjective(coinMat)

    property dualTolerance:
        def __get__(self):
            return self.CppSelf.dualTolerance()

        def __set__(self, value):
           self.CppSelf.setDualTolerance(value)

    property primalTolerance:
        def __get__(self):
            return self.CppSelf.primalTolerance()

        def __set__(self, value):
           self.CppSelf.setPrimalTolerance(value)

    property maxNumIteration:
        def __get__(self):
            return self.CppSelf.maximumIterations()

        def __set__(self, value):
           self.CppSelf.setMaxNumIteration(value)

    property logLevel:
        def __get__(self):
            return self.CppSelf.logLevel()

        def __set__(self, value):
            self.CppSelf.setLogLevel(value)

    property automaticScaling:
        def __get__(self):
            return self.CppSelf.automaticScaling()

        def __set__(self, value):
            self.CppSelf.setAutomaticScaling(value)

    property scaling:
        def __get__(self):
            return self.CppSelf.scalingFlag()
        def __set__(self, mode):
            self.CppSelf.scaling(mode)

    property infeasibilityCost:
        def __get__(self):
            return self.CppSelf.infeasibilityCost()
        def __set__(self, value):
            self.CppSelf.setInfeasibilityCost(value)


    property numberPrimalInfeasibilities:
        def __get__(self):
            return self.CppSelf.numberPrimalInfeasibilities()

    property optimizationDirection:
        def __get__(self):
            return ['ignore', 'min', 'max'][int(self.CppSelf.optimizationDirection())]
        def __set__(self, value):
            self.CppSelf.setOptimizationDirection({'ignore':0., 'min':1., 'max':-1.}[value])

    #############################################
    # get set
    #############################################

    def getRightHandSide(self, np.ndarray[np.double_t, ndim=1] rhs):
        '''
        Take a spare array, ``rhs``, and store the current right-hand-side
        in it.
        '''
        self.CppSelf.getRightHandSide(<double*>rhs.data)

    def getStatusCode(self):
        '''
        Get the probelm status as defined in CLP. Return value could be:

        * -1 - unknown e.g. before solve or if postSolve says not optimal
        * 0 - optimal
        * 1 - primal infeasible
        * 2 - dual infeasible
        * 3 - stopped on iterations or time
        * 4 - stopped due to errors
        * 5 - stopped by event handler (virtual int ClpEventHandler::event())

        '''
        return self.CppSelf.status()

    def getStatusString(self):
        '''
        Return the problem status in string using the code from
        :func:`getStatusCode`
        '''
        return problemStatus[self.getStatusCode()]

    def setColumnLower(self, ind, val):
        '''
        Set the lower bound of variable index ``ind`` to ``val``.
        '''
        self.CppSelf.setColumnLower(ind, val)

    def setColumnUpper(self, ind, val):
        '''
        Set the upper bound of variable index ``ind`` to ``val``.
        '''
        self.CppSelf.setColumnUpper(ind, val)

    def setRowLower(self, ind, val):
        '''
        Set the lower bound of constraint index ``ind`` to ``val``.
        '''
        self.CppSelf.setRowLower(ind, val)

    def setRowUpper(self, ind, val):
        '''
        Set the upper bound of constraint index ``ind`` to ``val``.
        '''
        self.CppSelf.setRowUpper(ind, val)

    def useCustomPrimal(self, customPrimal):
        '''
        Determines if
        :func:`cylp.python.pivot.PivotPythonBase.isPivotAcceptable`
        should be called just before each pivot is performed (right after the
        entering and leaving variables are obtained.
        '''
        self.CppSelf.useCustomPrimal(customPrimal)

    def getUseCustomPrimal(self):
        '''
        Return the value of ``useCustomPrimal``. See :func:`useCustomPrimal`.

        :rtype: int  :math:`\in \{0, 1\}`
        '''
        return self.CppSelf.getUseCustomPrimal()

    def flagged(self, varInd):
        '''
        Returns ``1`` if variable index ``varInd`` is flagged.

        :rtype: int  :math:`\in \{0, 1\}`
        '''
        return self.CppSelf.flagged(varInd)

    def setFlagged(self, varInd):
        '''
        Set variables index ``varInd`` flagged.
        '''
        self.CppSelf.setFlagged(varInd)

##    def currentDualTolerance(self):
##        return self.CppSelf.currentDualTolerance()
##
    def largestDualError(self):
        return self.CppSelf.largestDualError()

    def pivotRow(self):
        '''
        Return the index of the constraint corresponding to the (basic) leaving
        variable.

        :rtype: int
        '''
        return self.CppSelf.pivotRow()

    def setPivotRow(self, v):
        '''
        Set the ``v``\ 'th variable of the basis as the leaving variable.
        '''
        self.CppSelf.setPivotRow(v)

    def sequenceIn(self):
        '''
        Return the index of the entering variable.

        :rtype: int
        '''
        return self.CppSelf.sequenceIn()

    def setSequenceIn(self, v):
        '''
        Set the variable index ``v`` as the entering variable.
        '''
        self.CppSelf.setSequenceIn(v)

##    def dualTolerance(self):
##        '''
##        Return the dual tolerance.
##
##        :rtype: float
##        '''
##        return self.CppSelf.dualTolerance()

    cdef double* rowLower(self):
        '''
        Return the lower bounds of the constraints as a double*.
        This can be used only in Cython.
        '''
        return self.CppSelf.rowLower()

    cdef double* rowUpper(self):
        '''
        Return the upper bounds of the constraints as a double*.
        This can be used only in Cython.
        '''
        return self.CppSelf.rowUpper()

    def getVariableNames(self):
        '''
        Return the variable name. (e.g. that was set in the mps file)
        '''
        cdef vector[string] names = self.CppSelf.getVariableNames()
        ret = []
        for i in range(names.size()):
            ret.append(names[i].c_str())
        return ret

    cpdef setVariableName(self, varInd, name):
        '''
        Set the name of variable index ``varInd`` to ``name``.

        :arg varInd: variable index
        :type varInd: integer
        :arg name: desired name for the variable
        :type name: string

        '''
        self.CppSelf.setVariableName(varInd, name)

    cpdef setConstraintName(self, constInd, name):
        '''
        Set the name of constraint index ``constInd`` to ``name``.

        :arg constInd: constraint index
        :type constInd: integer
        :arg name: desired name for the constraint
        :type name: string

        '''
        self.CppSelf.setConstraintName(constInd, name)

    cdef int* pivotVariable(self):
        '''
        Return the index set of the basic variables.

        :rtype: int*
        '''
        return self.CppSelf.pivotVariable()

    cpdef  getPivotVariable(self):
        '''
        Return the index set of the basic variables.

        :rtype: Numpy array
        '''
        return <object>self.CppSelf.getPivotVariable()

    cpdef CLP_getVarStatus(self, int sequence):
        '''
        get the status of a variable

        * free : 0
        * basic : 1
        * atUpperBound : 2
        * atLowerBound : 3
        * superBasic : 4
        * fixed : 5

        :rtype: int
        '''
        return self.CppSelf.getStatus(sequence)

    cpdef CLP_setVarStatus(self, int sequence, int status):
        '''
        set the status of a variable

        * free : 0
        * basic : 1
        * atUpperBound : 2
        * atLowerBound : 3
        * superBasic : 4
        * fixed : 5

        '''
        self.CppSelf.setStatus(sequence, CLP_variableStatusEnum[status])

    def setVariableStatus(self, arg, status):
        '''
        Set the status of a variable.

        :arg arg: Specifies the variable to change (a CyLPVar, or an index)
        :type status: CyLPVar, int
        :arg status: 'basic', 'atUpperBound', 'atLowerBound', 'superBasic', 'fixed'
        :type status: string


        Example:

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex
        >>> s = CyClpSimplex()
        >>> x = s.addVariable('x', 4)
        >>> # Using CyLPVars:
        >>> s.setVariableStatus(x[1:3], 'basic')
        >>> s.getVariableStatus(x[1])
        'basic'
        >>> # Using a variable index directly
        >>> s.setVariableStatus(1, 'atLowerBound')
        >>> s.getVariableStatus(x[1])
        'atLowerBound'

        '''
        status = CLP_variableStatusEnum[StatusToInt[status]]
        if isinstance(arg, int):
            self.CppSelf.setStatus(arg, status)
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of setVarStatus can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            var = arg
            model = self.cyLPModel
            inds = model.inds
            varName = var.name
            if not inds.hasVar(varName):
                raise Exception('No such variable: %s' % varName)
            x = inds.varIndex[varName]
            if var.parent:
                for i in var.indices:
                    self.CppSelf.setStatus(x[i], status)
            else:
                for i in xrange(var.dim):
                    self.CppSelf.setStatus(x[i], status)

    def getVariableStatus(self, arg):
        '''
        Get the status of a variable.
        '''
        if isinstance(arg, int):
            return IntToStatus[self.CppSelf.getStatus(arg)]
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of getVarStatus can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            var = arg
            model = self.cyLPModel
            inds = model.inds
            varName = var.name
            if not inds.hasVar(varName):
                raise Exception('No such variable: %s' % varName)
            x = inds.varIndex[varName]
            if var.parent:
                s = np.array([IntToStatus[
                            self.CppSelf.getStatus(x[i])]
                            for i in var.indices])
            else:
                s = np.array([IntToStatus[
                            self.CppSelf.getStatus(x[i])]
                            for i in xrange(var.dim)])
            if len(s) == 1:
                return s[0]
            return s

    def setConstraintStatus(self, arg, status):
        '''
        Set the status of a constraint.

        :arg arg: Specifies the constraint to change (name or index)
        :type status: string,int
        :arg status: 'basic', 'atUpperBound', 'atLowerBound', 'superBasic', 'fixed'
        :type status: string

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex
        >>> s = CyClpSimplex()
        >>> x = s.addVariable('x', 4)
        >>> s.addConstraint(0 <= x[0] + x[1] <= 1, 'const1')
        >>> # Using constraint name:
        >>> s.setConstraintStatus('const1', 'atUpperBound')
        >>> s.getConstraintStatus('const1')
        'atUpperBound'
        >>> # Using constraint index directly
        >>> s.setConstraintStatus(0, 'atLowerBound')
        >>> s.getConstraintStatus('const1')
        'atLowerBound'
        '''
        status = CLP_variableStatusEnum[StatusToInt[status]]
        if isinstance(arg, int):
            arg += self.nVariables
            self.CppSelf.setStatus(arg, status)
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of setVarStatus can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            model = self.cyLPModel
            inds = model.inds
            constName = arg
            if not inds.hasConst(constName):
                raise Exception('No such constraint: %s' % constName)
            c = inds.constIndex[constName]
            cInds = c + self.nVariables
            for i in xrange(len(cInds)):
                self.CppSelf.setStatus(cInds[i], status)

    def getConstraintStatus(self, arg):
        '''
        Get the status of a constraint.
        '''
        if isinstance(arg, int):
            arg += self.nVariables
            return IntToStatus[self.CppSelf.getStatus(arg)]
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of setVarStatus can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            model = self.cyLPModel
            inds = model.inds
            constName = arg
            if not inds.hasConst(constName):
                raise Exception('No such constraint: %s' % constName)
            c = inds.constIndex[constName]
            cInds = c + self.nVariables
            s = np.array([IntToStatus[
                            self.CppSelf.getStatus(cInds[i])]
                            for i in xrange(len(cInds))])
            if len(s) == 1:
                return s[0]
            return s


    def setColumnUpperArray(self, np.ndarray[np.double_t, ndim=1] columnUpper):
        '''
        columnUpper should have n+m elements. The method only does
        a pointer assignment. If you only want to set the first n
        elements use setColumnUpperFirstElements().
        '''
        self.CppSelf.setColumnUpperArray(<double*>columnUpper.data)

    def setColumnUpperFirstElements(self, np.ndarray[np.double_t, ndim=1] columnUpper):
        '''
        Run a loop in C++ and set the first n elements of variables' upperbounds
        '''
        self.CppSelf.setColumnUpperFirstElements(len(columnUpper), <double*>columnUpper.data)

    def setColumnLowerArray(self, np.ndarray[np.double_t, ndim=1] columnLower):
        '''
        columnLower should have n+m elements. The method only does
        a pointer assignment. If you only want to set the first n
        elements use setColumnLowerFirstElements().
        '''
        self.CppSelf.setColumnLowerArray(<double*>columnLower.data)

    def setColumnLowerFirstElements(self, np.ndarray[np.double_t, ndim=1] columnLower):
        '''
        Run a loop in C++ and set the first n elements of variables' lowerbounds
        '''
        self.CppSelf.setColumnLowerFirstElements(len(columnLower), <double*>columnLower.data)

    def setColumnLowerSubset(self, np.ndarray[np.int32_t, ndim=1] indicesOfIndices,
                                   np.ndarray[np.int32_t, ndim=1] indices,
                                   np.ndarray[np.double_t, ndim=1] columnLower):
        '''
        This method is defined for a very specific purpose.
        It's only to be used to speed up self.addConstraint()
        '''
        self.CppSelf.setColumnLowerSubset(len(indicesOfIndices), <int*> indicesOfIndices.data,
                                          <int*> indices.data,
                                          <double*>columnLower.data)

    def setColumnUpperSubset(self, np.ndarray[np.int32_t, ndim=1] indicesOfIndices,
                                   np.ndarray[np.int32_t, ndim=1] indices,
                                   np.ndarray[np.double_t, ndim=1] columnUpper):
        '''
        This method is defined for a very specific purpose.
        It's only to be used to speed up self.addConstraint()
        '''
        self.CppSelf.setColumnUpperSubset(len(indicesOfIndices), <int*> indicesOfIndices.data,
                                          <int*> indices.data,
                                          <double*>columnUpper.data)

    def setRowUpperArray(self, np.ndarray[np.double_t, ndim=1] rowUpper):
        self.CppSelf.setRowUpperArray(<double*>rowUpper.data)

    def setRowLowerArray(self, np.ndarray[np.double_t, ndim=1] rowLower):
        self.CppSelf.setRowLowerArray(<double*>rowLower.data)

    def setObjectiveArray(self, np.ndarray[np.double_t, ndim=1] objective):
        self.CppSelf.setObjectiveArray(<double*>objective.data, len(objective))

    cdef double* primalColumnSolution(self):
        return self.CppSelf.primalColumnSolution()

    cdef double* dualColumnSolution(self):
        return self.CppSelf.dualColumnSolution()

    cdef double* primalRowSolution(self):
        return self.CppSelf.primalRowSolution()

    cdef double* dualRowSolution(self):
        return self.CppSelf.dualRowSolution()

    def CLP_dualConstraintSolution(self):
        cdef np.npy_intp shape[1]
        shape[0] = <np.npy_intp> self.nConstraints
        ndarray = np.PyArray_SimpleNewFromData(1, shape,
                                               np.NPY_DOUBLE, <void*> self.dualRowSolution())
        return ndarray

    #############################################
    # CLP Methods
    #############################################

    def initialSolve(self, presolve='on'):
        '''
        Run CLP's initialSolve. It does a presolve and uses primal or dual
        Simplex to solve a problem.

        **Usage example**

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getMpsExample
        >>> s = CyClpSimplex()
        >>> f = getMpsExample()
        >>> s.readMps(f)
        0
        >>> s.initialSolve()
        'optimal'
        >>> round(s.objectiveValue, 4)
        2520.5717

        '''
        presolve = 0 if presolve == 'on' and self.Hessian is None else 1
        return problemStatus[self.CppSelf.initialSolve(presolve)]

    def initialPrimalSolve(self):
        '''
        Run CLP's initalPrimalSolve. The same as :func:`initalSolve` but force
        the use of primal Simplex.

        **Usage example**

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getMpsExample
        >>> s = CyClpSimplex()
        >>> f = getMpsExample()
        >>> s.readMps(f)
        0
        >>> s.initialPrimalSolve()
        'optimal'
        >>> round(s.objectiveValue, 4)
        2520.5717

        '''
        return problemStatus[self.CppSelf.initialPrimalSolve()]

    def initialDualSolve(self):
        '''
        Run CLP's initalPrimalSolve. The same as :func:`initalSolve` but force
        the use of dual Simplex.

        **Usage example**

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getMpsExample
        >>> s = CyClpSimplex()
        >>> f = getMpsExample()
        >>> s.readMps(f)
        0
        >>> s.initialDualSolve()
        'optimal'
        >>> round(s.objectiveValue, 4)
        2520.5717

        '''
        return problemStatus[self.CppSelf.initialDualSolve()]

    def __iadd__(self, cons):
        self.addConstraint(cons)
        return self

    def addConstraint(self, cons, name='', addMpsNames=True):
        '''
        Adds constraints ``cons``  to the problem. Example for the value
        of ``cons`` is ``0 <= A * x <= b`` where ``A`` is a Numpy matrix and
        b is a :py:class:`CyLPArray`.
        '''
        if self.cyLPModel:
            m = self.cyLPModel
            nVarsBefore = m.nVars
            nConsBefore = m.nCons
            c = m.addConstraint(cons, name, addMpsNames)

            # If the dimension is changing, load from scratch
            if nConsBefore == 0 or m.nVars - nVarsBefore != 0:
                self.loadFromCyLPModel(self.cyLPModel)

            # If the constraint to be added is just a variable range
            elif c.isRange:
                var = c.variables[0]
                dim = var.parentDim
                varinds = m.inds.varIndex[var.name]

                lb = var.parent.lower if var.parent else var.lower
                ub = var.parent.upper if var.parent else var.upper

                #if len(var.indices != 0 :
                self.setColumnLowerSubset(var.indices,
                                              varinds,
                                              lb)
                self.setColumnUpperSubset(var.indices,
                                              varinds,
                                              ub)
                #for i in var.indices:
                #    self.setColumnLower(varinds[i], lb[i])
                #    self.setColumnUpper(varinds[i], ub[i])

            # If the constraint is a "real" constraint, but no
            # dimension changes required
            else:
                mainCoef = None
                for varName in m.varNames:
                    dim = m.pvdims[varName]
                    coef = sparse.coo_matrix((c.nRows, dim))
                    keys = [k for k in c.varCoefs.keys() if k.name == varName]
                    for var in keys:
                        coef = coef + c.varCoefs[var]
                    mainCoef = sparseConcat(mainCoef, coef, 'h')

                self.addConstraints(c.nRows,
                        c.lower, c.upper, mainCoef.indptr,
                        mainCoef.indices, mainCoef.data)
        else:
            raise Exception('To add a constraint you must set ' \
                            'cylpSimplex.cyLPModel first.')

    def removeConstraint(self, name):
        '''
        Removes constraint named ``name`` from the problem.
        '''
        if self.cyLPModel:
            indsOfRemovedConstriants = self.cyLPModel.removeConstraint(name)
            self.CLP_deleteConstraints(indsOfRemovedConstriants)
            #self.loadFromCyLPModel(self.cyLPModel)
        else:
            raise Exception('To remove a constraint you must set ' \
                            'cylpSimplex.cyLPModel first.')

    def addVariable(self, varname, dim, isInt=False):
        '''
        Add variable ``var`` to the problem.
        '''
        if not self.cyLPModel:
            self.cyLPModel = CyLPModel()
        var = self.cyLPModel.addVariable(varname, dim, isInt)
        self.loadFromCyLPModel(self.cyLPModel)
        return var
        #else:
        #    raise Exception('To add a variable you must set ' \
        #                    'cylpSimplex.cyLPModel first.')

    def removeVariable(self, name):
        '''
        Removes variable named ``name`` from the problem.
        '''
        if self.cyLPModel:
            self.cyLPModel.removeVariable(name)
            self.loadFromCyLPModel(self.cyLPModel)
        else:
            raise Exception('To remove a variable you must set ' \
                            'cylpSimplex.cyLPModel first.')

    def getVarByName(self, name):
        if not self.cyLPModel:
            raise Exception('No cylpSimplex.cyLPModel is set.')
        return self.cyLPModel.getVarByName(name)

    def getVarNameByIndex(self, ind):
        if not self.cyLPModel:
            raise Exception('No cylpSimplex.cyLPModel is set.')
        return self.cyLPModel.inds.reverseVarSearch(ind)

    def CLP_addConstraint(self, numberInRow,
                    np.ndarray[np.int32_t, ndim=1] columns,
                    np.ndarray[np.double_t, ndim=1] elements,
                    rowLower,
                    rowUpper):
        '''
        Add a constraint to the problem, CLP style. See CLP documentation.
        Not commonly used in cylp.
        For cylp modeling tool see :mod:`cylp.python.modeling.CyLPModel`.
        '''
        # TODO: This makes adding a row real slower,
        # but it is better than a COIN EXCEPTION!
        if (columns >= self.nVariables).any():
            raise Exception('CyClpSimplex.pyx:addConstraint: Column ' \
                    'index out of range (number of columns: ' \
                                '%d)' % (self.nVariables))
        self.CppSelf.addRow(numberInRow, <int*>columns.data,
                            <double*>elements.data, rowLower, rowUpper)

    def CLP_deleteConstraints(self, np.ndarray[np.int32_t, ndim=1] which):
        '''
        Delete constraints indexed by ``which`` from the LP.
        '''
        if (which >= self.nConstraints).any():
            raise Exception('CyClpSimplex.pyx:deleteConstraints: Constraint ' \
                    'index out of range (number of constraints: ' \
                                '%d)' % (self.nConstraints))
        self.CppSelf.deleteRows(len(which), <int*>which.data)

    def CLP_deleteVariables(self, np.ndarray[np.int32_t, ndim=1] which):
        '''
        Delete variables indexed by ``which`` from the LP.
        '''
        if (which >= self.nVariables).any():
            raise Exception('CyClpSimplex.pyx:deleteVariables: variable ' \
                    'index out of range (number of variables: ' \
                                '%d)' % (self.nVariables))
        self.CppSelf.deleteColumns(len(which), <int*>which.data)

    def CLP_addVariable(self, numberInColumn,
                        np.ndarray[np.int32_t, ndim=1] rows,
                        np.ndarray[np.double_t, ndim=1] elements,
                        columnLower,
                        columnUpper,
                        objective):
        '''
        Add a variable to the problem, CLP style. See CLP documentation.
        For cylp modeling tool see :mod:`cylp.python.modeling.CyLPModel`.
        '''
        # TODO: This makes adding a column real slower,
        # but it is better than a COIN EXCEPTION!
        if (rows >= self.nConstraints).any():
            raise Exception('CyClpSimplex.pyx:addColumn: Row '\
                    'index out of range (number of rows:  ' \
                        '%d)' % (self.nConstraints))
        self.CppSelf.addColumn(numberInColumn, <int*>rows.data,
                <double*> elements.data, columnLower,
                               columnUpper, objective)

    def addVariables(self, number,
                        np.ndarray[np.double_t, ndim=1] columnLower,
                        np.ndarray[np.double_t, ndim=1] columnUpper,
                        np.ndarray[np.double_t, ndim=1] objective,
                        np.ndarray[np.int32_t, ndim=1] columnStarts,
                        np.ndarray[np.int32_t, ndim=1] rows,
                        np.ndarray[np.double_t, ndim=1] elements):
        '''
        Add ``number`` variables at once, CLP style.
        For cylp modeling tool see :mod:`cylp.python.modeling.CyLPModel`.
        '''
        self.CppSelf.addColumns(number, <double*>columnLower.data,
                                        <double*>columnUpper.data,
                                        <double*>objective.data,
                                        <int*>columnStarts.data,
                                        <int*>rows.data,
                                        <double*>elements.data)

    def addConstraints(self, number,
                        np.ndarray[np.double_t, ndim=1] rowLower,
                        np.ndarray[np.double_t, ndim=1] rowUpper,
                        np.ndarray[np.int32_t, ndim=1] rowStarts,
                        np.ndarray[np.int32_t, ndim=1] columns,
                        np.ndarray[np.double_t, ndim=1] elements):
        '''
        Add ``number`` constraints at once, CLP style.
        For cylp modeling tool see :mod:`cylp.python.modeling.CyLPModel`.
        '''
        self.CppSelf.addRows(number, <double*>rowLower.data,
                                    <double*>rowUpper.data,
                                    <int*>rowStarts.data,
                                    <int*>columns.data,
                                    <double*>elements.data)

    cpdef int readMps(self, filename, int keepNames=False,
            int ignoreErrors=False) except *:
        '''
        Read an mps file. See this :ref:`modeling example <modeling-usage>`.
        '''
        filename = filename.encode('ascii')
        name, ext = os.path.splitext(filename)
        if ext not in [b'.mps', b'.qps']:
            print('unrecognised extension %s' % ext)
            return -1

        if ext == b'.mps':
            return self.CppSelf.readMps(filename, keepNames, ignoreErrors)
        else:
            m = CyCoinMpsIO.CyCoinMpsIO()
            ret = m.readMps(filename)
            self._Hessian = m.Hessian

            # Since CyCoinMpsIO.readMps seems to be different from ClpModle.readMps
            # for the moment we read the problem again
            # FIXME: should be fixed
            #self.loadProblem(m.matrixByCol, m.variableLower, m.variableUpper,
            #                 m.objCoefficients,
            #                 m.constraintLower, m.constraintUpper)
            #return ret

            return self.CppSelf.readMps(filename, keepNames, ignoreErrors)


    def extractCyLPModel(self, fileName, keepNames=False, ignoreErrors=False):
        if self.readMps(fileName, keepNames, ignoreErrors) != 0:
            return None
        m = CyLPModel()

        x = m.addVariable('x', self.nVariables)

        # Copy is crucial. Memory space should be different than
        # that of Clp. Else, a resize will ruin these.
        c_up = CyLPArray(self.constraintsUpper).copy()
        c_low = CyLPArray(self.constraintsLower).copy()

        mat = self.matrix
        C = csc_matrixPlus((mat.elements, mat.indices, mat.vectorStarts),
                             shape=(self.nConstraints, self.nVariables))

        m += c_low <= C * x <= c_up

        x_up = CyLPArray(self.variablesUpper).copy()
        x_low = CyLPArray(self.variablesLower).copy()

        m += x_low <= x <= x_up

        m.objective = self.objective

        self.cyLPModel = m
        return m

    def _extractStartFinish(self, startFinishOptions):
        if isinstance(startFinishOptions, int):
            sf = startFinishOptions
        else:
            sf = 0
            for option in startFinishOptions:
                sf = sf | startFinishOptionsDic[option]
        return sf

    def primal(self, ifValuesPass=0, startFinishOptions=0, presolve=False):
        '''
        Solve the problem using the primal simplex algorithm.
        See this :ref:`usage example <simple-run>`.

        startFinishOptions is a string containing one or
        more of the following characters:
        'x': do not delete work areas
        'f': use old factorization if possible
        's': skip initialization of work areas
        So one might call ``self.primal(startFinishOptions='sx')``
        '''
        sf = self._extractStartFinish(startFinishOptions)
        if presolve:
            return self.primalWithPresolve()
        else:
            return problemStatus[self.CppSelf.primal(
                             ifValuesPass, sf)]

    def dual(self, ifValuesPass=0, startFinishOptions=0, presolve=False):
        '''
        Runs CLP dual simplex.

        **Usage Example**

        >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getMpsExample
        >>> s = CyClpSimplex()
        >>> f = getMpsExample()
        >>> s.readMps(f)
        0
        >>> s.dual()
        'optimal'

        '''
        sf = self._extractStartFinish(startFinishOptions)
        if presolve:
            return self.dualWithPresolve()
        else:
            return problemStatus[self.CppSelf.dual(
                            ifValuesPass, sf)]

    def setPerturbation(self, value):
        '''
        Perturb the problem by ``value``.
        '''
        self.CppSelf.setPerturbation(value)

    cdef setPrimalColumnPivotAlgorithm(self, void* choice):
        '''
        Set primal simplex's pivot rule to ``choice``
        This is used when setting a pivot rule in Cython
        '''
        cdef CppClpPrimalColumnPivot* c = <CppClpPrimalColumnPivot*> choice
        self.CppSelf.setPrimalColumnPivotAlgorithm(c)

    cdef setDualRowPivotAlgorithm(self, void* choice):
        '''
        Set dual simplex's pivot rule to ``choice``
        This is used when setting a pivot rule in Cython
        '''
        cdef CppClpDualRowPivot* c = <CppClpDualRowPivot*> choice
        self.CppSelf.setDualRowPivotAlgorithm(c)

    def resize(self, newNumberRows, newNumberColumns):
        '''
        Resize the problem. After a call to ``resize`` the problem will have
        ``newNumberRows`` constraints and ``newNumberColumns`` variables.
        '''
        self.CppSelf.resize(newNumberRows, newNumberColumns)

    def getBInvACol(self, col, np.ndarray[np.double_t, ndim=1] cl):
        '''
        Compute :math:`A_B^{-1}A_{col}` and store the result in ``cl``.
        '''
        self.CppSelf.getBInvACol(col, <double*>cl.data)

    def getBInvCol(self, col, np.ndarray[np.double_t, ndim=1] cl):
        '''
        Return :math:`A_B^{-1}_{col}` and store the result in ``cl``.
        '''
        self.CppSelf.getBInvCol(col, <double*>cl.data)

    def transposeTimes(self, scalar, CyCoinIndexedVector x,
                       CyCoinIndexedVector y, CyCoinIndexedVector z):
        '''
        Compute :math:`x * scalar * A + y` and store the result in ``z``.
        '''
        self.CppSelf.transposeTimes(self.CppSelf, scalar,
                                    x.CppSelf, y.CppSelf, z.CppSelf)

    def transposeTimesSubset(self, number,
                             np.ndarray[np.int64_t, ndim=1] which,
                             np.ndarray[np.double_t, ndim=1] pi,
                             np.ndarray[np.double_t, ndim=1] y):
        '''
        Compute :math:`y_{which} - pi^{T}A_{which}` where ``which`` is a
        variable index set. Store the result in ``y``.
        '''
        self.CppSelf.transposeTimesSubset(number, <int*>which.data,
                                          <double*>pi.data, <double*>y.data)

    def transposeTimesSubsetAll(self,
                             np.ndarray[np.int64_t, ndim=1] which,
                             np.ndarray[np.double_t, ndim=1] pi,
                             np.ndarray[np.double_t, ndim=1] y):
        '''
        Same as :func:`transposeTimesSubset` but here ``which``
        can also address slack variables.
        '''
        self.CppSelf.transposeTimesSubsetAll(len(which),
                                            <long long int*>which.data,
                                            <double*>pi.data,
                                            <double*>y.data)

    def isInteger(self, ind):
        '''
        Returns True if the variable index ``ind`` is integer.
        '''
        return self.CppSelf.isInteger(ind)

    def setInteger(self, arg):
        '''
        if ``arg`` is an integer: mark variable index ``arg`` as integer.
        if ``arg`` is a :class:`CyLPVar` object: mark variable
        ``arg`` as integer. Here is an example of the latter:

        >>> import numpy as np
        >>> from cylp.cy import CyClpSimplex
        >>> from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray
        >>> model = CyLPModel()
        >>>
        >>> x = model.addVariable('x', 3)
        >>> y = model.addVariable('y', 2)
        >>>
        >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
        >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
        >>> D = np.matrix([[1., 2.],[0, 1]])
        >>> a = CyLPArray([5, 2.5])
        >>> b = CyLPArray([4.2, 3])
        >>> x_u= CyLPArray([2., 3.5])
        >>>
        >>> model += A*x <= a
        >>> model += 2 <= B * x + D * y <= b
        >>> model += y >= 0
        >>> model += 1.1 <= x[1:3] <= x_u
        >>>
        >>> c = CyLPArray([1., -2., 3.])
        >>> model.objective = c * x + 2 * y.sum()
        >>>
        >>>
        >>> s = CyClpSimplex(model)
        >>> s.setInteger(x[1:3])
        >>>
        >>> cbcModel = s.getCbcModel()
        >>> cbcModel.solve()
        0
        >>> print(cbcModel.status)
        'solution'
        >>>
        >>> sol_x = cbcModel.primalVariableSolution['x']
        >>> (abs(sol_x -
        ...     np.array([0.5, 2, 2]) ) <= 10**-6).all()
        True
        >>> sol_y = cbcModel.primalVariableSolution['y']
        >>> (abs(sol_y -
        ...     np.array([0, 0.75]) ) <= 10**-6).all()
        True

        '''

        if isinstance(arg, int):
            self.CppSelf.setInteger(arg)
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of setInteger can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            var = arg
            model = self.cyLPModel
            inds = model.inds
            varName = var.name
            if not inds.hasVar(varName):
                raise Exception('No such variable: %s' % varName)
            x = inds.varIndex[varName]
            if var.parent:
                for i in var.indices:
                    self.CppSelf.setInteger(x[i])
            else:
                for i in xrange(var.dim):
                    self.CppSelf.setInteger(x[i])


    def copyInIntegerInformation(self, np.ndarray[np.uint8_t, ndim=1] colType):
        '''
        Take in a character array containing 0-1 specifying whether or not
        a variable is integer
        '''
        self.CppSelf.copyInIntegerInformation(<char*>colType.data)

    def replaceMatrix(self, CyCoinPackedMatrix matrix, deleteCurrent=False):
        self.CppSelf.replaceMatrix(matrix.CppSelf, deleteCurrent)

    def loadQuadraticObjective(self, CyCoinPackedMatrix matrix):
        self.CppSelf.loadQuadraticObjective(matrix.CppSelf)

    def preSolve(self, feasibilityTolerance=0.0,
                 keepIntegers=0, numberPasses=5,
                 dropNames=0, doRowObjective=0):
        cdef CppIClpSimplex* model = self.CppSelf.preSolve(self.CppSelf,
                                feasibilityTolerance, keepIntegers,
                                numberPasses, dropNames, doRowObjective)
        s = CyClpSimplex()
        if model == NULL:
            print("Presolve says problem infeasible.")
            return s

        s.setCppSelf(model)
        return s
        #self.setCppSelf(model)

    def postSolve(self, updateStatus=True):
        self.CppSelf.postSolve(updateStatus)

    def dualWithPresolve(self, feasibilityTolerance=0.0,
                 keepIntegers=0, numberPasses=5,
                 dropNames=0, doRowObjective=0):
        ret = self.CppSelf.dualWithPresolve(self.CppSelf,
                                feasibilityTolerance, keepIntegers,
                                numberPasses, dropNames, doRowObjective)
        if ret == -2000:
            print("Presolve says problem infeasible.")
            return -2000

        return problemStatus[ret]

    def primalWithPresolve(self, feasibilityTolerance=0.0,
                 keepIntegers=0, numberPasses=5,
                 dropNames=0, doRowObjective=0):
        ret = self.CppSelf.primalWithPresolve(self.CppSelf,
                                feasibilityTolerance, keepIntegers,
                                numberPasses, dropNames, doRowObjective)
        if ret == -2000:
            print("Presolve says problem infeasible.")
            return -2000

        return problemStatus[ret]

    def writeMps(self, filename, formatType=0, numberAcross=2, objSense=0):
        try:
            f = open(filename, 'w')
            f.close()
        except:
            raise Exception('No write access for %s or an intermediate \
                            directory does not exist.' % filename)

        m = self.cyLPModel
        if m:
            inds = m.inds
            for var in m.variables:
                varinds = inds.varIndex[var.name]
                for i in xrange(var.dim):
                    self.setVariableName(varinds[i], var.mpsNames[i])

            for con in m.constraints:
                coninds = inds.constIndex[con.name]
                for i in xrange(con.nRows):
                    self.setConstraintName(coninds[i], con.mpsNames[i])
        return self.CppSelf.writeMps(filename, formatType, numberAcross,
                                     objSense)

    def writeLp(self, filename, extension="", epsilon=10**-5, numberAcross=10,
                        decimals=5, objSense=0.0, useRowNames=1):
        try:
            f = open(filename, 'w')
            f.close()
        except:
            raise Exception('No write access for %s or an intermediate \
                            directory does not exist.' % filename)

        m = self.cyLPModel
        if m:
            inds = m.inds
            for var in m.variables:
                varinds = inds.varIndex[var.name]
                for i in xrange(var.dim):
                    self.setVariableName(varinds[i], var.mpsNames[i])

            for con in m.constraints:
                coninds = inds.constIndex[con.name]
                for i in xrange(con.nRows):
                    self.setConstraintName(coninds[i], con.mpsNames[i])
        self.CppSelf.writeLp(filename, extension, epsilon, numberAcross, decimals, objSense, useRowNames)

    def readLp(self, char *filename, epsilon=10**-5):
        return self.CppSelf.readLp(filename, epsilon)

    def updateColumnFT(self, CyCoinIndexedVector spare, CyCoinIndexedVector updatedColumn):
        return self.CppSelf.updateColumnFT(spare.CppSelf, updatedColumn.CppSelf)

    def updateColumnTranspose(self, CyCoinIndexedVector regionSparse1,
                                    CyCoinIndexedVector regionSparse2):
        return self.CppSelf.updateColumnTranspose(regionSparse1.CppSelf, regionSparse2.CppSelf)

    #############################################
    # Modeling
    #############################################

    def loadFromCyLPModel(self, cyLPModel):
        '''
        Set the coefficient matrix, constraint bounds, and variable
        bounds based on the data in *cyLPModel* which should be and object
        of *CyLPModel* class.

        This method is usually called from CyClpSimplex's constructor.
        But in a case that the CyClpSimplex instance is created before
        we have the CyLPModel we use this method to load the LP,
        for example:

        >>> import numpy as np
        >>> from cylp.cy.CyClpSimplex import CyClpSimplex, getModelExample
        >>>
        >>> s = CyClpSimplex()
        >>> model = getModelExample()
        >>> s.loadFromCyLPModel(model)
        >>>
        >>> s.primal()
        'optimal'
        >>> sol_x = s.primalVariableSolution['x']
        >>> (abs(sol_x -
        ...     np.array([0.2, 2, 1.1]) ) <= 10**-6).all()
        True

        '''
        self.cyLPModel = cyLPModel
        (mat, constraintLower, constraintUpper,
                    variableLower, variableUpper) = cyLPModel.makeMatrices()

        n = len(variableLower)
        m = len(constraintLower)
        if n == 0:# or m == 0:
            return

        self.resize(m, n)
        if mat is not None:
            if not isinstance(mat, sparse.coo_matrix):
                mat = mat.tocoo()

            coinMat = CyCoinPackedMatrix(True, np.array(mat.row, np.int32),
                                        np.array(mat.col, np.int32),
                                        np.array(mat.data, np.double))
        else:
            coinMat = CyCoinPackedMatrix(True, np.array([], np.int32),
                                        np.array([], np.int32),
                                        np.array([], np.double))
        self.replaceMatrix(coinMat, True)

        #start adding the arrays and the matrix to the problem

        for i in xrange(n):
            self.setColumnLower(i, variableLower[i])
            self.setColumnUpper(i, variableUpper[i])

        for i in xrange(m):
            self.setRowLower(i, constraintLower[i])
            self.setRowUpper(i, constraintUpper[i])

        #setting integer informations
        variables = cyLPModel.variables
        curVarInd = 0
        for var in variables:
            if var.isInt:
                for i in xrange(curVarInd, curVarInd + var.dim):
                    self.setInteger(i)
            curVarInd += var.dim


        if cyLPModel.objective is not None:
            self.objective = cyLPModel.objective

    def evaluateAt(self, x0):
        '''
        Evaluate the objective function at x0
        '''
        if self.Hessian is not None:
            return (np.dot(self.objectiveCoefficients, x0) +
                    0.5 * np.dot(x0, self.Hessian.dot(x0)) - self.objectiveOffset)
        else:
            return np.dot(self.objectiveCoefficients, x0) - self.objectiveOffset

    def gradientAt(self, x0):
        if self.Hessian is not None:
            return self.objectiveCoefficients + self.Hessian * x0
        else:
            return self.objectiveCoefficients


    #############################################
    # Integer Programming
    #############################################

    def getCbcModel(self):
        '''
        Run initialSolve, return a :class:`CyCbcModel` object that can be
        used to add cuts, run B&B and ...
        '''
        cdef CppICbcModel* model = self.CppSelf.getICbcModel()
        cm =  CyCbcModel()
        cm.setCppSelf(model)
        cm.setClpModel(self)
        if self.cyLPModel:
            cm.cyLPModel = self.cyLPModel
        return cm

    #############################################
    # cylp and Pivoting
    #############################################

    def isPivotAcceptable(self):
        return (<CyPivotPythonBase>
                self.cyPivot).pivotMethodObject.isPivotAcceptable()

    def checkVar(self, i):
        (<CyPivotPythonBase>self.cyPivot).pivotMethodObject.checkVar(i)
        return (<CyPivotPythonBase>self.cyPivot).pivotMethodObject.checkVar(i)

    def setPrimalColumnPivotAlgorithmToWolfe(self):
        '''
        Set primal simplex's pivot rule to the Cython implementation of
        Wolfe's rule used to solve QPs.
        '''
        cdef CyWolfePivot wp = CyWolfePivot()
        self.setPrimalColumnPivotAlgorithm(wp.CppSelf)

    def setPrimalColumnPivotAlgorithmToPE(self):
        '''
        Set primal simplex's pivot rule to the Cython
        implementation of *positive edge*
        '''
        cdef CyPEPivot pe = CyPEPivot()
        self.setPrimalColumnPivotAlgorithm(pe.CppSelf)

    def setPivotMethod(self, pivotMethodObject):
        '''
        Takes a python object and sets it as the primal
        simplex pivot rule. ``pivotObjectMethod`` should
        implement :py:class:`PivotPythonBase`.
        See :ref:`how to use custom Python pivots
        to solve LPs <custom-pivot-usage>`.
        '''
        if not issubclass(pivotMethodObject.__class__, PivotPythonBase):
            raise TypeError('pivotMethodObject should be of a \
                            class derived from PivotPythonBase')

        cdef CyPivotPythonBase p = CyPivotPythonBase(pivotMethodObject)
        self.cyPivot = p
        p.cyModel = self
        self.setPrimalColumnPivotAlgorithm(p.CppSelf)

    def setDualPivotMethod(self, dualPivotMethodObject):
        '''
        Takes a python object and sets it as the dual
        pivot rule. ``dualPivotObjectMethod`` should
        implement :py:class:`DualPivotPythonBase`.
        See :ref:`how to use custom dual Python pivots
        to solve LPs <custom-dual-pivot-usage>`.       '''
        if not issubclass(dualPivotMethodObject.__class__, DualPivotPythonBase):
            raise TypeError('dualPivotMethodObject should be of a \
                            class derived from DualPivotPythonBase')

        cdef CyDualPivotPythonBase p = CyDualPivotPythonBase(dualPivotMethodObject)
        self.cyDualPivot = p
        p.cyModel = self
        self.setDualRowPivotAlgorithm(p.CppSelf)


    cpdef filterVars(self,  inds):
        return <object>self.CppSelf.filterVars(<PyObject*>inds)

    def setObjectiveCoefficient(self, elementIndex, elementValue):
        '''
        Set the objective coefficients using sparse vector elements
        ``elementIndex`` and ``elementValue``.
        '''
        self.CppSelf.setObjectiveCoefficient(elementIndex, elementValue)

    def partialPricing(self, start, end,
                      np.ndarray[np.int32_t, ndim=1] numberWanted):
        '''
        Perform partial pricing from variable ``start`` to variable ``end``.
        Stop when ``numberWanted`` variables good variable checked.
        '''
        return self.CppSelf.partialPrice(start, end, <int*>numberWanted.data)

    def setComplementarityList(self, np.ndarray[np.int32_t, ndim=1] cl):
        self.CppSelf.setComplementarityList(<int*>cl.data)

    cpdef getACol(self, int ncol, CyCoinIndexedVector colArray):
        '''
        Gets column ``ncol`` of ``A`` and store it in ``colArray``.
        '''
        self.CppSelf.getACol(ncol, colArray.CppSelf)

    cpdef vectorTimesB_1(self, CyCoinIndexedVector vec):
        '''
        Compute :math:`vec A_B^{-1}` and store it in ``vec``.
        '''
        self.CppSelf.vectorTimesB_1(vec.CppSelf)

    cdef primalRow(self, CppCoinIndexedVector * rowArray,
                                       CppCoinIndexedVector * rhsArray,
                                       CppCoinIndexedVector * spareArray,
                                       CppCoinIndexedVector * spareArray2,
                                       int valuesPass):
        raise Exception('CyClpPrimalColumnPivotBase.pyx: pivot column ' \
                        'should be implemented.')

    def argWeightedMax(self, arr, arr_ind, w, w_ind):
        return self.CppSelf.argWeightedMax(<PyObject*>arr, <PyObject*>arr_ind,
                                            <PyObject*>w, <PyObject*>w_ind)

#    def getnff(self):
#        status = self.status
#        return np.where((status & 7 != 5) & (status & 64 == 0))[0]
#
#    def getfs(self):
#        status = self.status
#        return np.where((status & 7 == 4) | (status & 7 == 0))[0]

    cdef int* ComplementarityList(self):
        return self.CppSelf.ComplementarityList()

    cpdef getComplementarityList(self):
        return <object>self.CppSelf.getComplementarityList()

    def setComplement(self, var1, var2):
        '''
        Set ``var1`` as the complementary variable of ``var2``. These
        arguments may be integers signifying indices, or CyLPVars.
        '''

        if isinstance(var1, int) and isinstance(var2, int) :
           self.CppSelf.setComplement(var1, var2)
        elif True:  # isinstance(arg, CyLPVar):
            if self.cyLPModel is None:
                raise Exception('The argument of setInteger can be ' \
                                'a CyLPVar only if the object is built ' \
                                'using a CyLPModel.')
            if var1.dim != var2.dim:
                raise Exception('Variables should have the same  ' \
                                'dimensions to be complements.' \
                                ' Got %s: %g and %s: %g' %
                                (var1.name, var1.dim, var2.name, var2.dim))

            model = self.cyLPModel
            inds = model.inds
            vn1 = var1.name
            vn2 = var2.name

            if not inds.hasVar(vn1):
                raise Exception('No such variable: %s' % vn1)
            x1 = inds.varIndex[vn1]
            if not inds.hasVar(vn2):
                raise Exception('No such variable: %s' % vn2)
            x2 = inds.varIndex[vn2]

            for i in xrange(var1.dim):
                self.CppSelf.setComplement(x1[i], x2[i])

#    def setComplement(self, var1, var2):
#        'sets var1 and var2 to be complements'
#        #When you create LP using CoinModel getComplementarityList
#        #cannot return with the right size
#        #cl = self.getComplementarityList()
#        #print(var1, var2, len(cl))
#        #cl[var1], cl[var2] = var2, var1
#        self.CppSelf.setComplement(var1, var2)

    def loadProblemFromCyCoinModel(self, CyCoinModel modelObject, int
                                        tryPlusMinusOne=False):
        return self.CppSelf.loadProblem(modelObject.CppSelf, tryPlusMinusOne)

    def loadProblem(self, CyCoinPackedMatrix matrix,
                 np.ndarray[np.double_t, ndim=1] collb,
                 np.ndarray[np.double_t, ndim=1] colub,
                 np.ndarray[np.double_t, ndim=1] obj,
                 np.ndarray[np.double_t, ndim=1] rowlb,
                 np.ndarray[np.double_t, ndim=1] rowub,
                 np.ndarray[np.double_t, ndim=1] rowObjective=np.array([])):
        cdef double* rd
        if len(rowObjective) == 0:
            rd = NULL
        else:
            rd = <double*> rowObjective.data
        self.CppSelf.loadProblem(matrix.CppSelf, <double*> collb.data,
                                         <double*> colub.data,
                                         <double*> obj.data,
                                         <double*> rowlb.data,
                                         <double*> rowub.data,
                                         <double*> rd)

    def getCoinInfinity(self):
        return self.CppSelf.getCoinInfinity()

    #############################################
    # Osi
    #############################################

    def setBasisStatus(self, np.ndarray[np.int32_t, ndim=1] cstat,
                             np.ndarray[np.int32_t, ndim=1] rstat):
        self.CppSelf.setBasisStatus(<int*>cstat.data, <int*>rstat.data)

    def getBasisStatus(self):
        cdef np.ndarray[np.int32_t, ndim=1] cstat = \
                                np.zeros(self.nVariables, dtype='int32')
        cdef np.ndarray[np.int32_t, ndim=1] rstat = \
                                np.zeros(self.nConstraints, dtype='int32')
        self.CppSelf.getBasisStatus(<int*>cstat.data, <int*>rstat.data)
        return cstat, rstat


def getModelExample():
    '''
    Return a model example to be used in doctests.
    '''
    import numpy as np
    from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray
    from cylp.cy import CyClpSimplex

    model = CyLPModel()
    x = model.addVariable('x', 3)
    y = model.addVariable('y', 2)

    A = np.matrix([[1., 2., 0], [1., 0, 1.]])
    B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    D = np.matrix([[1., 2.], [0, 1]])
    a = CyLPArray([5, 2.5])
    b = CyLPArray([4.2, 3])
    x_u= CyLPArray([2., 3.5])

    model += A * x <= a
    model += 2 <= B * x + D * y <= b
    model += y >= 0
    model += 1.1 <= x[1:3] <= x_u

    c = CyLPArray([1., -2., 3.])
    model.objective = c * x + 2 * y.sum()

    return model


cpdef cydot(CyCoinIndexedVector v1, CyCoinIndexedVector v2):
    return cdot(v1.CppSelf, v2.CppSelf)


def getMpsExample():
    '''
    Return full path to an MPS example file for doctests
    '''
    import os
    import inspect
    curpath = os.path.dirname(inspect.getfile(inspect.currentframe()))
    return os.path.join(curpath, '../input/p0033.mps')


cdef int RunIsPivotAcceptable(void * ptr):
    cdef CyClpSimplex CyWrapper = <CyClpSimplex>(ptr)
    return CyWrapper.isPivotAcceptable()


cdef int RunVarSelCriteria(void * ptr, int varInd):
    cdef CyClpSimplex CyWrapper = <CyClpSimplex>(ptr)
    return CyWrapper.checkVar(varInd)


cdef class VarStatus:
    free = 0
    basic = 1
    atUpperBound = 2
    atLowerBound = 3
    superBasic = 4
    fixed = 5
    status_ = np.array([free,
                        basic,
                        atUpperBound,
                        atLowerBound,
                        superBasic,
                        fixed])



# cython: embedsignature=True

import numpy as np
cimport numpy as np
from cylp.cy cimport CyPEPivot
#from CyClpSimplex cimport CyClpSimplex
#cimport cython.operator.dereference as deref

cdef class CyPEPivot(CyClpPrimalColumnPivotBase):
    def __init__(self, cyModel):
        self.cyModel = cyModel
        CyClpPrimalColumnPivotBase.__init__(self)

    cdef pivotColumn(self, CppCoinIndexedVector* cppupdates, CppCoinIndexedVector* cppspareRow1,
                    CppCoinIndexedVector* cppspareRow2, CppCoinIndexedVector* cppspareCol1,
                    CppCoinIndexedVector* cppspareCol2):
        updates = CyCoinIndexedVector()
        updates.setCppSelf(cppupdates)
        spareRow1 = CyCoinIndexedVector()
        spareRow1.setCppSelf(cppspareRow1)
        spareRow2 = CyCoinIndexedVector()
        spareRow2.setCppSelf(cppspareRow2)
        spareCol1 = CyCoinIndexedVector()
        spareCol1.setCppSelf(cppspareCol1)
        spareCol2 = CyCoinIndexedVector()
        spareCol2.setCppSelf(cppspareCol2)

        s = self.cyModel

        # Update the reduced costs, for both the original and the slack variables
        if updates.nElements:
            s.updateColumnTranspose(spareRow2, updates)
            s.transposeTimes(-1, updates, spareCol2, spareCol1)
            s.reducedCosts[s.nVariables:][updates.indices] -= updates.elements[:updates.nElements]
            s.reducedCosts[:s.nVariables][spareCol1.indices] -= spareCol1.elements[:spareCol1.nElements]
        updates.clear()
        spareCol1.clear()

        rc = s.reducedCosts

        cdef double tol = s.dualTolerance

        indicesToConsider = np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree))[0]

        rc2 = abs(rc[indicesToConsider])

        cdef double maxRc = -1
        cdef int maxInd = -1
        cdef double maxCompRc = -1
        cdef int maxCompInd = -1
        cdef int ind
        for i in xrange(len(rc2)):
            ind = indicesToConsider[i]
            if rc2[i] > maxCompRc:
                if self.isCompatible(ind):
                    maxCompRc = rc2[i]
                    maxCompInd = ind
                else:
                    pass
                if rc2[i] > maxRc:
                    maxRc = rc2[i]
                    maxInd = ind

        if maxCompInd != -1 and maxCompRc > 0.4 * maxRc:
            print(s.iteration, ' : comp selected', maxCompInd)
            return maxCompInd

        print(s.iteration, ' : incomp selected', maxInd)
        self.updateP()
        self.updateW()
        #print('updated')
        return maxInd

    cdef CyClpPrimalColumnPivot* clone(self, bint copyData):
        cdef CyClpPrimalColumnPivot* ret =  \
                    <CyClpPrimalColumnPivot*> new CppClpPrimalColumnPivotBase(
                            <cpy_ref.PyObject*>self,
                            <runPivotColumn_t>RunPivotColumn,
                            <runClone_t>RunClone,
                            <runSaveWeights_t>RunSaveWeights)
        return ret
    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex * model, int mode):
        self.CppSelf.setModel(model)



'''
As a part of ``cylp.python.pivots`` it implements Dantzig's
 Simplex dual pivot rule. Although it already exists in CLP,
for testing purposes we implement one in Python.
'''

import sys
import numpy as np
from operator import itemgetter
from random import shuffle
from math import floor
from .DualPivotPythonBase import DualPivotPythonBase
#from cylp.py.pivots import DantzigPivot


class DualDantzigPivot(DualPivotPythonBase):
    '''
    Dantzig's dual pivot rule implementation.

    .. _custom-dual-pivot-usage:

    **Usage**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.pivots.DualDantzigPivot import DualDantzigPivot
    >>> from cylp.py.pivots.DualDantzigPivot import getMpsExample
    >>> # Get the path to a sample mps file
    >>> f = getMpsExample()
    >>> s = CyClpSimplex()
    >>> s.readMps(f)  # Returns 0 if OK
    0
    >>> pivot = DualDantzigPivot(s)
    >>> s.setDualPivotMethod(pivot)
    >>> s.dual()
    'optimal'
    >>> round(s.objectiveValue, 5)
    2520.57174

    '''

    def __init__(self, clpModel):
        self.dim = clpModel.nRows + clpModel.nCols
        self.clpModel = clpModel

    def pivotRow(self):
        model = self.clpModel
        nConstraints = model.nConstraints
        basicVarInds = model.basicVariables

        u = model.upper[basicVarInds]
        l = model.lower[basicVarInds]
        s = model.solution[basicVarInds]
        infeasibilities = np.maximum(s - u, l - s)

        m = max(infeasibilities)

        if m > model.primalTolerance:
            return np.argmax(infeasibilities)
        return -1

    def updateWeights(self, inp, spare, spare2, updatedColumn):
        model = self.clpModel
        pr = model.pivotRow()
        model.updateColumnFT(spare, updatedColumn)
        indices = updatedColumn.indices
        elements = updatedColumn.elements
        if updatedColumn.isInPackedMode:
            if pr in indices:
                ind = np.where(indices==pr)[0][0]
                return elements[ind]
        else:
            return elements[pr]
        return 0

    def updatePrimalSolution(self, primalUpdate, primalRatio, objectiveChange):
        model = self.clpModel
        nConstraints = model.nConstraints
        basicVarInds = model.basicVariables
        rowNumbers = primalUpdate.indices
        elements = primalUpdate.elements
        nElements = primalUpdate.nElements
        changeObj = 0

        sol = model.solution[basicVarInds[rowNumbers]]
        cost = model.cost[basicVarInds[rowNumbers]]

        if primalUpdate.isInPackedMode:
            change = primalRatio * elements[:nElements]
            model.solution[basicVarInds[rowNumbers]] -= change
        else:
            change = primalRatio * elements[rowNumbers]
            model.solution[basicVarInds[rowNumbers]] -= change

        changeObj = -np.dot(change, cost)
        primalUpdate.clear()
        objectiveChange[0] += changeObj

        return changeObj


def getMpsExample():
    import os
    import inspect
    import sys
    cylpDir = os.environ['CYLP_SOURCE_DIR']
    return os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')


if __name__ == "__main__":
    print(sys.argv)
    if len(sys.argv) == 1:
        import doctest
        doctest.testmod()
    else:
        from cylp.cy import CyClpSimplex
        from cylp.py.pivots import DualDantzigPivot
        s = CyClpSimplex()
        s.readMps(sys.argv[1])  # Returns 0 if OK
        pivot = DualDantzigPivot(s)
        s.setDualPivotMethod(pivot)
        s.dual()



from cylp.cy.CyClpSimplex cimport CyClpSimplex, CppIClpSimplex

cdef extern from "ClpSimplex.hpp":
    cdef cppclass CppClpSimplex "ClpSimplex":
        pass

cdef extern from "OsiClpSolverInterface.hpp":
    cdef cppclass CppOsiClpSolverInterface "OsiClpSolverInterface":
        CppClpSimplex * getModelPtr()

cdef extern from "OsiSolverInterface.hpp":
    cdef cppclass CppOsiSolverInterface "OsiSolverInterface":
        pass

cdef class CyOsiSolverInterface:
    cdef CppOsiSolverInterface* CppSelf
    cdef setCppSelf(self, CppOsiSolverInterface* s)



cdef extern from *:
    CppOsiClpSolverInterface* dynamic_cast_osi_2_clp "dynamic_cast<OsiClpSolverInterface*>" (CppOsiSolverInterface*)
    CppIClpSimplex* static_cast_clp_2_iclp "static_cast<IClpSimplex*>" (CppClpSimplex*)

cdef class CyOsiSolverInterface:
    'CyOsiSolverInterface documentation'
    def __cinit__(self):
        pass

    cdef setCppSelf(self, CppOsiSolverInterface* s):
        del self.CppSelf
        self.CppSelf = s

    property clpModel:
        def __get__(self):
            cdef CyClpSimplex s = CyClpSimplex()
            cdef CppOsiClpSolverInterface* si = dynamic_cast_osi_2_clp(self.CppSelf)
            cdef CppIClpSimplex* sp = static_cast_clp_2_iclp(si.getModelPtr())
            s.setCppSelf(sp)
            return s



import sys
from cylp.cy.CyTest import CySolve

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Arg error: Usage example: python CySolve.py lp.mps d')
        sys.exit(0)
    method = sys.argv[2]
    if method == 'p':
        print('Positive edge is not implemented in Cython yet. Quitting.')
        sys.exit(1)
    CySolve(sys.argv[1], sys.argv[2])



# cython: embedsignature=True

cdef class CyCutGeneratorPythonBase(CyCglCutGeneratorBase):
    def __init__(self, cutGeneratorObject, cyLPModel=None):
        CyCglCutGeneratorBase.__init__(self)
        self.cutGeneratorObject = cutGeneratorObject
        Py_INCREF(cutGeneratorObject)
        self.cyLPModel = cyLPModel

    def __dealloc__(self):
        Py_DECREF(self.cutGeneratorObject)

    cdef generateCuts(self, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info):
        cysi =  CyOsiSolverInterface()
        cysi.setCppSelf(si)
        cycs = CyOsiCuts()
        cycs.setCppSelf(cs)

        cyinfo = CyCglTreeInfo()
        cyinfo.setCppSelf(&info)
        cuts = self.cutGeneratorObject.generateCuts(cysi, cyinfo)
        if type(cuts) is not list:
            cuts = [cuts]
        for cut in cuts:
            # Getting a CyLPConstraint
            isRange = cut.evaluate('cut').isRange
            if isRange:
                cycs.addColumnCut(cut, self.cyLPModel)
            else:
                cycs.addRowCut(cut, self.cyLPModel)

    cdef CppCglCutGenerator* clone(self):
        cdef CppCglCutGenerator* ret =  \
                <CppCglCutGenerator*> new CppCglCutGeneratorBase(
                            <cpy_ref.PyObject*>self,
                            <runGenerateCuts_t>RunGenerateCuts,
                            <runCglClone_t>RunCglClone)
        return ret





# cython: embedsignature=True

from itertools import product
from cylp.py.mip import NodeCompareBase
from cylp.py.modeling.CyLPModel import CyLPSolution
from cylp.cy.CyCutGeneratorPythonBase cimport CyCutGeneratorPythonBase
from libcpp cimport bool


cdef int RunTest(void* ptr, CppICbcNode*x, CppICbcNode*y):
    obj = <object> ptr
    return obj.compare(CyCbcNode().setCppSelf(x),
                       CyCbcNode().setCppSelf(y))

cdef bool RunNewSolution(void* ptr, CppICbcModel* model,
        double objectiveAtContinuous,
        int numberInfeasibilitiesAtContinuous):
    obj = <object> ptr
    return obj.newSolution(CyCbcModel().setCppSelf(model),
                                objectiveAtContinuous,
                                numberInfeasibilitiesAtContinuous)

cdef int RunEvery1000Nodes(void* ptr, CppICbcModel* model, int numberNodes):
    obj = <object> ptr
    return obj.every1000Nodes(CyCbcModel().setCppSelf(model),
                                 numberNodes)

# Understandable messages to translate what branchAndBound() returns
problemStatus =  ['search completed', 'relaxation infeasible',
         'stopped on gap', 'stopped on nodes', 'stopped on time',
         'stopped on user event', 'stopped on solutions',
         'linear relaxation unbounded', 'unset']

cdef class CyCbcModel:
    '''
    Interfaces ``CbcModel``. To solve a first you create a
    :class:`cylp.cy.CyClpSimplex` object either
    by reading it from an ``mps`` file using
    :func:`CyClpSimplex.readMps() <cylp.cy.CyClpSimplex.CyClpSimplex.readMps>`
    or by using cylp modeling tool
    :mod:`cylp.py.modeling.CyLPModel`. Then you ask the object for a
    ``CyCbcModel`` which is capable solving MIPs using B&B

    **Usage**

    >>> import numpy as np
    >>> from cylp.cy import CyCbcModel, CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray
    >>> model = CyLPModel()
    >>>
    >>> x = model.addVariable('x', 3, isInt=True)
    >>> y = model.addVariable('y', 2)
    >>>
    >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
    >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    >>> D = np.matrix([[1., 2.],[0, 1]])
    >>> a = CyLPArray([5, 2.5])
    >>> b = CyLPArray([4.2, 3])
    >>> x_u= CyLPArray([2., 3.5])
    >>>
    >>> model += A*x <= a
    >>> model += 2 <= B * x + D * y <= b
    >>> model += y >= 0
    >>> model += 1.1 <= x[1:3] <= x_u
    >>>
    >>> c = CyLPArray([1., -2., 3.])
    >>> model.objective = c * x + 2 * y.sum()
    >>>
    >>> s = CyClpSimplex(model)
    >>>
    >>> cbcModel = s.getCbcModel()
    >>>
    >>> cbcModel.solve()
    0
    >>> print (cbcModel.status)
    'solution'
    >>> sol_x = cbcModel.primalVariableSolution['x']
    >>>
    >>> (abs(sol_x - np.array([0, 2, 2])) <= 10**-6).all()
    True

    '''

    def __cinit__(self, cyLPModel=None):
        self.cyLPModel = cyLPModel
        self.cutGenerators = []

    def __dealloc__(self):
        for generator in self.cutGenerators:
            Py_DECREF(generator)

        try:
            if self.CppSelf:
                del self.CppSelf
        except AttributeError:
            pass

    cdef setCppSelf(self, CppICbcModel* cppmodel):
        self.CppSelf = cppmodel
        return self

    cdef setClpModel(self, clpmodel):
        self.clpModel = clpmodel
        return self

    def setNodeCompare(self, nodeCompareObject):
        if not isinstance(nodeCompareObject, NodeCompareBase):
            raise TypeError('setNodeCompare argument should be a ' \
                        'NodeCompareBase object. Got %s' %
                        nodeCompareObject.__class__)

        self.CppSelf.setNodeCompare(<PyObject*>nodeCompareObject,
                                    RunTest, RunNewSolution, RunEvery1000Nodes)

    cpdef addCutGenerator(self, CyCglCutGenerator generator,
                        howOften=1, name="", normal=True, atSolution=False,
                        infeasible=False, howOftenInSub=-100, whatDepth=-1,
                        whatDepthInSub=-1):
        self.cutGenerators.append(generator)
        Py_INCREF(generator)
        if isinstance(name, str):
            # Cast strings/unicode to bytes
            name = name.encode('utf-8')
        self.CppSelf.addCutGenerator(generator.CppSelf, howOften,
                                    name, normal, atSolution,
                                    infeasible, howOftenInSub, whatDepth,
                                    whatDepthInSub)

    def addPythonCutGenerator(self, pythonCutGeneratorObject,
                        howOften=1, name="", normal=True, atSolution=False,
                        infeasible=False, howOftenInSub=-100, whatDepth=-1,
                        whatDepthInSub=-1):
        cdef CyCutGeneratorPythonBase generator = \
                            CyCutGeneratorPythonBase(pythonCutGeneratorObject)
        generator.cyLPModel = self.cyLPModel
        self.CppSelf.addCutGenerator(<CppCglCutGenerator*>generator.CppSelf,
                                    howOften, name, normal, atSolution,
                                    infeasible, howOftenInSub, whatDepth,
                                    whatDepthInSub)

    def solve(self):
        '''
        Call CbcMain. Solve the problem using the same parameters used by CbcSolver.
        Equivalent to solving the model from the command line using cbc's binary.
        '''
        return self.CppSelf.cbcMain()

    property status:
        def __get__(self):
            # secondaryStatus() should be used instead of status() (??)
            if self.isRelaxationInfeasible():
               return problemStatus[1]
            if self.isRelaxationAbandoned():
               return 'relaxation abandoned'
            if self.CppSelf.isProvenInfeasible():
               return 'problem proven infeasible'
            if self.CppSelf.isProvenOptimal():
               return 'solution'
            return problemStatus[self.CppSelf.secondaryStatus()]

    property logLevel:
        def __get__(self):
            return self.CppSelf.logLevel()

        def __set__(self, value):
            self.CppSelf.setLogLevel(value)

    def isRelaxationInfeasible(self):
        return self.CppSelf.isInitialSolveProvenPrimalInfeasible()

    def isRelaxationDualInfeasible(self):
        return self.CppSelf.isInitialSolveProvenDualInfeasible()

    def isRelaxationOptimal(self):
        return self.CppSelf.isInitialSolveProvenOptimal()

    def isRelaxationAbandoned(self):
        return self.CppSelf.isInitialSolveAbandoned()

    property osiSolverInteface:
        def __get__(self):
            cdef CyOsiSolverInterface osi = CyOsiSolverInterface()
            osi.setCppSelf(self.CppSelf.solver())
            return osi

    property primalVariableSolution:
        def __get__(self):
            ret = <object>self.CppSelf.getPrimalVariableSolution()
            if self.cyLPModel:
                m = self.cyLPModel
                inds = m.inds
                d = {}
                for v in inds.varIndex.keys():
                    d[v] = ret[inds.varIndex[v]]
                    var = m.getVarByName(v)
                    if var.dims:
                        d[v] = CyLPSolution()
                        dimRanges = [range(i) for i in var.dims]
                        for element in product(*dimRanges):
                            d[v][element] = ret[var.__getitem__(element).indices[0]]
                ret = d
            else:
                names = self.clpModel.variableNames
                if names:
                    d = CyLPSolution()
                    for i in range(len(names)):
                        d[names[i]] = ret[i]
                    ret = d
            return ret

    property solutionCount:
        def __get__(self):
            return self.CppSelf.getSolutionCount()

    property numberHeuristicSolutions:
        def __get__(self):
            return self.CppSelf.getNumberHeuristicSolutions()

    property nodeCount:
        def __get__(self):
            return self.CppSelf.getNodeCount()

    property objectiveValue:
        def __get__(self):
            return self.CppSelf.getObjValue()

    property bestPossibleObjValue:
        def __get__(self):
            return self.CppSelf.getBestPossibleObjValue()

    property numberObjects:
        def __get__(self):
            return self.CppSelf.numberObjects()

    property integerTolerance:
        def __get__(self):
            return self.CppSelf.getIntegerTolerance()

        def __set__(self, value):
           self.CppSelf.setIntegerTolerance(value)

    property maximumSeconds:
        def __get__(self):
            return self.CppSelf.getMaximumSeconds()

        def __set__(self, value):
           self.CppSelf.setMaximumSeconds(value)

    property maximumNodes:
        def __get__(self):
            return self.CppSelf.getMaximumNodes()

        def __set__(self, value):
           self.CppSelf.setMaximumNodes(value)

    property numberThreads:
        def __get__(self):
            return self.CppSelf.getNumberThreads()

        def __set__(self, value):
            self.CppSelf.setNumberThreads(value)

    property allowableGap:
        def __get__(self):
            return self.CppSelf.getAllowableGap()

        def __set__(self, value):
            self.CppSelf.setAllowableGap(value)

    property allowableFractionGap:
        def __get__(self):
            return self.CppSelf.getAllowableFractionGap()

        def __set__(self, value):
            self.CppSelf.setAllowableFractionGap(value)

    property allowablePercentageGap:
        def __get__(self):
            return self.CppSelf.getAllowablePercentageGap()

        def __set__(self, value):
            self.CppSelf.setAllowablePercentageGap(value)

    property maximumSolutions:
        def __get__(self):
            return self.CppSelf.getMaximumSolutions()

        def __set__(self, value):
            self.CppSelf.setMaximumSolutions(value)



    #TODO: add access to solver: getLower, getUpper,...



from cython.operator cimport dereference as deref
cimport cpython.ref as cpy_ref
from cpython.ref cimport PyObject
from cylp.cy cimport CyClpSimplex
from cylp.cy.CyOsiCuts cimport CppOsiCuts
from cylp.cy.CyOsiSolverInterface cimport CppOsiSolverInterface
from cylp.cy.CyCglTreeInfo cimport CppCglTreeInfo
from cpython cimport Py_INCREF, Py_DECREF

#import numpy as np
#cimport numpy as np

cdef extern from "CglCutGenerator.hpp":
    cdef cppclass CppCglCutGenerator "CglCutGenerator":
        pass
    #CppCglCutGenerator* new_CyCglCutGenerator \
    #                                "new ICglCutGenerator" ()

cdef extern from "ICglCutGeneratorBase.h":
    #cdef cppclass CoinIndexedVector:
    #   pass
    ctypedef int (*runGenerateCuts_t)(void *obj,
                                     CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info)

    ctypedef CppCglCutGenerator* (*runCglClone_t)(void *obj)

    cdef cppclass CppCglCutGeneratorBase:

        CppCglCutGeneratorBase(PyObject* obj,
                                    runGenerateCuts_t runGenerateCuts,
                                    runCglClone_t runCglClone)

        void generateCuts(CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info)

        CppCglCutGenerator* clone()


cdef void RunGenerateCuts(void *ptr, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info)

cdef CppCglCutGenerator* RunCglClone(void *ptr)

cdef class CyCglCutGeneratorBase:
    cdef CppCglCutGeneratorBase* CppSelf
    cdef CyClpSimplex.CyClpSimplex cyModel
    #cpdef cyModel
    cdef generateCuts(self, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info)
    cdef CppCglCutGenerator * clone(self)




from cpython.ref cimport PyObject

cdef extern from "ICoinIndexedVector.hpp":
    cdef cppclass CppCoinIndexedVector "ICoinIndexedVector":
        int getNumElements()
        PyObject* getIndicesNPArray()
        PyObject* getDenseVectorNPArray()
        double & operator[](int)
        double getItem(int n)
        void setItem(int n, double value)
        void insert(int ind, double element)
        void reserve(int n)
        void clear()
        void empty()
        void assign(PyObject*, PyObject*)
        void Print()
        bint packedMode()
        void setNumElements(int value)
    CppCoinIndexedVector *new_CppCoinIndexedVector "new ICoinIndexedVector" ()


cdef class CyCoinIndexedVector:
    cdef CppCoinIndexedVector *CppSelf
    cpdef reserve(self, n)
    cpdef assign(self, ind, other)
    cdef setCppSelf(self, CppCoinIndexedVector* s)
    #cpdef getIndices(self)
    #cpdef getElements(self)
    #cpdef getNumElements(self)



# cython: profile=True
# cython: embedsignature=True

cimport cylp.cy.CyClpPrimalColumnPivotBase
#from cylp.cy import CyClpSimplex

cdef int RunPivotColumn(void *ptr, CppCoinIndexedVector* v1,
                        CppCoinIndexedVector* v2,
                        CppCoinIndexedVector* v3,
                        CppCoinIndexedVector* v4,
                        CppCoinIndexedVector* v5):
    return (<CyClpPrimalColumnPivotBase>(ptr)).pivotColumn(v1, v2, v3, v4, v5)

cdef CyClpPrimalColumnPivot* RunClone(void *ptr, bint copyData):
    return (<CyClpPrimalColumnPivotBase>(ptr)).clone(copyData)

cdef void RunSaveWeights(void *ptr,
                         CyClpSimplex.CppIClpSimplex * model,
                         int mode):
    (<CyClpPrimalColumnPivotBase>(ptr)).saveWeights(model, mode)


cdef class CyClpPrimalColumnPivotBase:
    def __init__(self):
        self.CppSelf = new CppClpPrimalColumnPivotBase(
            <cpy_ref.PyObject*>self,
            <runPivotColumn_t>RunPivotColumn,
            <runClone_t>RunClone,
            <runSaveWeights_t>RunSaveWeights)
        Py_INCREF(self)

    def __dealloc__(self):
        Py_DECREF(self)
        del self.CppSelf

    cdef pivotColumn(self, CppCoinIndexedVector* v1, CppCoinIndexedVector* v2,
                        CppCoinIndexedVector* v3, CppCoinIndexedVector* v4,
                        CppCoinIndexedVector* v5):
        raise Exception('CyClpPrimalColumnPivotBase.pyx: pivotColumn must' \
                        ' be implemented.')

    cdef CyClpPrimalColumnPivot* clone(self, bint copyData):
        cdef CyClpPrimalColumnPivot* ret =  \
                <CyClpPrimalColumnPivot*> new CppClpPrimalColumnPivotBase(
                                            <cpy_ref.PyObject*>self,
                                            <runPivotColumn_t>RunPivotColumn,
                                            <runClone_t>RunClone,
                                            <runSaveWeights_t>RunSaveWeights)
        return ret

    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex * model, int mode):
        raise Exception('CyClpPrimalColumnPivotBase.pyx: saveWeights must ' \
                        'be implemented.')

    cdef CyClpSimplex.CppIClpSimplex* model(self):
        return self.CppSelf.model()

    cdef void setModel(self, CyClpSimplex.CppIClpSimplex* m):
        self.CppSelf.setModel(m)

    cdef double* getReducedCosts(self):
        return self.model().djRegion()

    property nRows:
        def __get__(self):
            return self.CppSelf.model().getNumRows()

    property nCols:
        def __get__(self):
            return self.CppSelf.model().getNumCols()



from .CyLPModel import CyLPArray
from .CyLPModel import CyLPExpr
from .CyLPModel import CyLPVar
from .CyLPModel import IndexFactory
from .CyLPModel import CyLPConstraint
#from cylpTypes import CyLPVar
#from cylpTypes import CyLPArray
#from cylpTypes import CyLPExpr
#from cylpTypes import getCoinInfinity
from .CyLPModel import CyLPModel



from cylp.cy.CyClpPrimalColumnPivotBase cimport *
from cylp.cy cimport CyClpSimplex
from cylp.cy.CyCoinIndexedVector cimport CyCoinIndexedVector


cdef class CyWolfePivot(CyClpPrimalColumnPivotBase):
    cdef pivotColumn(self, CppCoinIndexedVector* v1,
                     CppCoinIndexedVector* v2, CppCoinIndexedVector* v3,
                     CppCoinIndexedVector* v4, CppCoinIndexedVector* v5)
    cdef CyClpPrimalColumnPivot* clone(self, bint copyData)
    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex* model, int mode)



'''
sparseUtil provides useful sparse matrix operations for incremental
construction of large sparse matrices from its block elements. Class
:class:`csc_matrixPlus` derives from ``sparse.csc_matrix`` and makes it
possible to set elements outside the matrix dimensions by resizing
the matrix whenever necessary. Class :class:`csr_matrixPlus` does
the same thing with ``sparse.csr_matrix``.

Function :py:func:`sparseConcat` concatenates two sparse matrices
regardless of their dimension alignments. Fills with zeros where necessary.

'''
from scipy import sparse
import numpy as np

class csc_matrixPlus(sparse.csc_matrix):
    def __init__(self, arg1, shape=None, dtype=None,
                 copy=False, fromMatrix=None):
        sparse.csc_matrix.__init__(self, arg1, shape, dtype, copy)
        if fromMatrix:
            self.__dict__.update(fromMatrix.__dict__)
        from cylp.py.modeling import CyLPExpr as CyLPExpr
        self.CyLPExpr = CyLPExpr
        self.rowScaleFactor = self.colScaleFactor = None

    def __setitem__(self, location, val):
        '''
        Set the item in row ``i`` and column ``j`` to ``val``.
        Increases matrix's size if necessary

        **Usage**

        >>> from cylp.py.utils.sparseUtil import csc_matrixPlus
        >>> import numpy as np
        >>> indptr = np.array([0, 2, 3, 6])
        >>> indices = np.array([0, 2, 2, 0, 1, 2])
        >>> data = np.array([1, 2, 3, 4, 5, 6])
        >>> s = csc_matrixPlus((data, indices, indptr), shape=(3, 3))
        >>> s[2, 5] = 11
        >>> s.todense()
        matrix([[ 1,  0,  4,  0,  0,  0],
                [ 0,  0,  5,  0,  0,  0],
                [ 2,  3,  6,  0,  0, 11]])

        '''
        iRow, iCol = location
        if not isinstance(val, (int, float)):
            return sparse.csc_matrix.__setitem__(self, (iRow, iCol), val)

        nCols = self.shape[1]

        #update shape if nec.
        if iRow >= self.shape[0]:
            self._shape = (iRow + 1, self.shape[1])

        if iCol < nCols:
            for i in range(self.indptr[iCol], self.indptr[iCol + 1]):
                if self.indices[i] == iRow:
                    self.data[i] = val
                    return
            #If we reach here it means that index does NOT exist
            for i in range(iCol + 1, nCols + 1):
                self.indptr[i] += 1
            indexOfElement = self.indptr[iCol + 1] - 1
            #If indices is empty
            if indexOfElement == 0:
                self.indices = np.array([iRow], dtype=np.int32)
                self.data = np.array([val])
            else:
                self.indices = np.concatenate((self.indices[:indexOfElement],
                                        np.array([iRow], dtype=np.int32),
                                        self.indices[indexOfElement:]), axis=0)

                self.data = np.concatenate((self.data[:indexOfElement],
                                        np.array([val]),
                                        self.data[indexOfElement:]), axis=0)

        else:
            #We don't have enough columns, increase dimension 1
            self.addColumns(iCol - nCols + 1)
            self.indptr[iCol + 1] += 1
            self.indices = np.concatenate((self.indices, np.array([iRow],
                dtype=np.int32)), axis=0)
            self.data = np.concatenate((self.data, np.array([val],
                dtype=np.int32)), axis=0)
            self._shape = (self._shape[0], iCol + 1)

    def addColumns(self, nCol):
        '''
        Add ``nCol`` columns to the matrix

        **Usage**

        >>> from cylp.py.utils.sparseUtil import csc_matrixPlus
        >>> s = csc_matrixPlus.getMatrixForTest()
        >>> s.shape
        (3, 3)
        >>> s.addColumns(3)
        >>> s.shape
        (3, 6)

        '''

        nElement = len(self.data)
        a = np.array(nCol * [nElement], dtype=np.int32)
        self.indptr = np.concatenate((self.indptr, a), axis=0)
        self._shape = (self._shape[0], self.shape[1] + nCol)

    def __getitem__(self, key):
        ret = sparse.csc_matrix.__getitem__(self, key)
        if isinstance(ret, (int, float)):
            return ret
        # This seems to cause some potential problems when the result is 1x1
        # It should really be returned as an int/float in that case, but
        # this prevents it and causes behavior to be different than
        # sparse.csr_matrix
        return csc_matrixPlus(ret)

    def row_scale(self, scaleFactor=None):
        data = self.data
        m = self.tocoo()
        irow, jcol = m.row, m.col
        if scaleFactor is not None:
            data /= scaleFactor[irow]
            self.rowScaleFactor = scaleFactor
            return scaleFactor

        rowScaleFactor = np.zeros(self.shape[0], dtype=np.float)
        for k in range(len(data)):
            row = irow[k]
            val = abs(data[k])
            rowScaleFactor[row] = max(rowScaleFactor[row], val)
        rowScaleFactor[rowScaleFactor == 0.0] = 1.0
        data /= rowScaleFactor[irow]
        self.rowScaleFactor = rowScaleFactor
        return rowScaleFactor


    def col_scale(self, scaleFactor=None):
        data = self.data
        m = self.tocoo()
        irow, jcol = m.row, m.col
        if scaleFactor is not None:
            data /= scaleFactor[jcol]
            self.colScaleFactor = scaleFactor
            return scaleFactor

        colScaleFactor = np.zeros(self.shape[1], dtype=np.float)

        for k in range(len(data)):
            col = jcol[k]
            val = abs(data[k])
            colScaleFactor[col] = max(colScaleFactor[col], val)
        colScaleFactor[colScaleFactor == 0.0] = 1.0
        data /= colScaleFactor[jcol]
        self.colScaleFactor = colScaleFactor
        return colScaleFactor

    def col_unscale(self, scaleFactor=None):
        if scaleFactor is None:
            scaleFactor = self.colScaleFactor
        if scaleFactor is not None:
            jcol = self.tocoo().col
            self.data *= scaleFactor[jcol]

    def row_unscale(self, scaleFactor=None):
        if scaleFactor is None:
            scaleFactor = self.rowScaleFactor
        if scaleFactor is not None:
            irow = self.tocoo().row
            self.data *= scaleFactor[irow]

    @property
    def T(self):
        return csr_matrixPlus(sparse.csc_matrix.transpose(self))

    def __le__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__le__(self, other)

    def __ge__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__ge__(self, other)

    def __mul__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__mul__(self, other)

    def __rmul__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__rmul__(self, other)

    def __add__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__add__(self, other)

    def __radd__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__radd__(self, other)

    def __rsub__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__rsub__(self, other)

    def __sub__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csc_matrix.__sub__(self, other)

    @staticmethod
    def getMatrixForTest():
        from cylp.py.utils.sparseUtil import csr_matrixPlus
        import numpy as np
        indptr = np.array([0, 2, 3, 6])
        indices = np.array([0, 2, 2, 0, 1, 2])
        data = np.array([1, 2, 3, 4, 5, 6])
        s = csc_matrixPlus((data, indices, indptr), shape=(3, 3))
        return s


class csr_matrixPlus(sparse.csr_matrix):
    def __init__(self, arg1, shape=None, dtype=None,
                 copy=False, fromMatrix=None):
        sparse.csr_matrix.__init__(self, arg1, shape, dtype, copy)
        if fromMatrix:
            self.__dict__.update(fromMatrix.__dict__)
        from cylp.py.modeling import CyLPExpr as CyLPExpr
        self.CyLPExpr = CyLPExpr
        self.rowScaleFactor = self.colScaleFactor = None

    def __setitem__(self, location, val):
        '''
        Sets the item in row ``i`` and col ``j`` to ``val``.
        Increases matrix's ``shape[1]`` if necessary

        **Usage**

        >>> from cylp.py.utils.sparseUtil import csr_matrixPlus
        >>> import numpy as np
        >>> indptr = np.array([0, 2, 3, 6])
        >>> indices = np.array([0, 2, 2, 0, 1, 2])
        >>> data = np.array([1, 2, 3, 4, 5, 6])
        >>> s = csr_matrixPlus((data, indices, indptr), shape=(3, 3))
        >>> s[5,2] = 11
        >>> print(s.todense())
        [[ 1  0  2]
         [ 0  0  3]
         [ 4  5  6]
         [ 0  0  0]
         [ 0  0  0]
         [ 0  0 11]]

        '''
        iRow, iCol = location
        if not isinstance(val, (int, float)):
            return sparse.csr_matrix.__setitem__(self, (iRow, iCol), val)

        nRows = self.shape[0]
        nCols = self.shape[1]

        l = self.tolil()

        if iCol >= nCols:
            l._shape = (nRows, iCol + 1)
            nCols = iCol + 1

        if iRow >= nRows:
            l._shape = (iRow + 1, nCols)
            nRowsToAdd = iRow + 1 - nRows
            l_temp = sparse.lil_matrix((nRowsToAdd, 1))
            l.data = np.concatenate((l.data, l_temp.data))
            l.rows = np.concatenate((l.rows, l_temp.rows))

        l[iRow, iCol] = val
        s = csr_matrixPlus(l)

        self._nnz = s.nnz
        self._shape = s._shape
        self.indices = s.indices
        self.indptr = s.indptr
        self.data = s.data
        self.has_sorted_indices = s.has_sorted_indices
        return

        nRows = self.shape[0]

        if iCol >= self.shape[1]:
            self._shape = (self.shape[0], iCol + 1)

        if iRow < nRows:
            for i in range(self.indptr[iRow], self.indptr[iRow + 1]):
                if self.indices[i] == iCol:
                    self.data[i] = val
                    return
            #if we reach here it means that index does NOT exist
            for i in range(iRow + 1, nRows + 1):
                self.indptr[i] += 1
            indexOfElement = self.indptr[iRow + 1] - 1
            # If indices is empty
            if indexOfElement == 0:
                self.indices = np.array([iCol], dtype=np.int32)
                self.data = np.array([val])
            else:
                self.indices = np.concatenate((self.indices[:indexOfElement],
                                        np.array([iCol], dtype=np.int32),
                                        self.indices[indexOfElement:]), axis=0)
                self.data = np.concatenate((self.data[:indexOfElement],
                                        np.array([val]),
                                        self.data[indexOfElement:]), axis=0)
        else:
            #We don't have enough columns, increase dimension 1
            self.addRows(iRow - nRows + 1)
            self.indptr[iRow + 1] += 1
            self.indices = np.concatenate((self.indices, np.array([iCol],
                dtype=np.int32)), axis=0)
            self.data = np.concatenate((self.data, np.array([val],
                dtype=np.int32)), axis=0)
            self._shape = (iRow + 1, self._shape[1])

    def addRows(self, nRow):
        '''
        Add ``nRow`` rows to the matrix

        **Usage**

        >>> from cylp.py.utils.sparseUtil import csr_matrixPlus
        >>> s = csr_matrixPlus.getMatrixForTest()
        >>> s.shape
        (3, 3)
        >>> s.addRows(2)
        >>> s.shape
        (5, 3)

        '''

        nElement = len(self.data)
        a = np.array(nRow * [nElement], dtype=np.int32)
        self.indptr = np.concatenate((self.indptr, a), axis=0)
        self._shape = (self._shape[0] + nRow, self.shape[1])

    def __getitem__(self, key):
        ret = sparse.csr_matrix.__getitem__(self, key)
        if isinstance(ret, (int, float)):
            return ret
        # This seems to cause some potential problems when the result is 1x1
        # It should really be returned as an int/float in that case, but
        # this prevents it and causes behavior to be different than
        # sparse.csr_matrix
        return csr_matrixPlus(ret)

    def row_scale(self, scaleFactor=None):
        data = self.data
        m = self.tocoo()
        irow, jcol = m.row, m.col
        if scaleFactor is not None:
            data /= scaleFactor[irow]
            self.rowScaleFactor = scaleFactor
            return scaleFactor

        rowScaleFactor = np.zeros(self.shape[0], dtype=np.float)
        for k in range(len(data)):
            row = irow[k]
            val = abs(data[k])
            rowScaleFactor[row] = max(rowScaleFactor[row], val)
        rowScaleFactor[rowScaleFactor == 0.0] = 1.0
        data /= rowScaleFactor[irow]
        self.rowScaleFactor = rowScaleFactor
        return rowScaleFactor


    def col_scale(self, scaleFactor=None):
        data = self.data
        m = self.tocoo()
        irow, jcol = m.row, m.col
        if scaleFactor is not None:
            data /= scaleFactor[jcol]
            self.colScaleFactor = scaleFactor
            return scaleFactor

        colScaleFactor = np.zeros(self.shape[1], dtype=np.float)

        for k in range(len(data)):
            col = jcol[k]
            val = abs(data[k])
            colScaleFactor[col] = max(colScaleFactor[col], val)
        colScaleFactor[colScaleFactor == 0.0] = 1.0
        data /= colScaleFactor[jcol]
        self.colScaleFactor = colScaleFactor
        return colScaleFactor

    def col_unscale(self, scaleFactor=None):
        if scaleFactor is None:
            scaleFactor = self.colScaleFactor
        if scaleFactor is not None:
            jcol = self.tocoo().col
            self.data *= scaleFactor[jcol]

    def row_unscale(self, scaleFactor=None):
        if scaleFactor is None:
            scaleFactor = self.rowScaleFactor
        if scaleFactor is not None:
            irow = self.tocoo().row
            self.data *= scaleFactor[irow]


    @property
    def T(self):
        return csc_matrixPlus(sparse.csr_matrix.transpose(self))

    def __le__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__le__(self, other)

    def __ge__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__ge__(self, other)

    def __mul__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__mul__(self, other)

    def __rmul__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__rmul__(self, other)

    def __add__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__add__(self, other)

    def __radd__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__radd__(self, other)

    def __rsub__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__rsub__(self, other)

    def __sub__(self, other):
        if isinstance(other, self.CyLPExpr):
            return NotImplemented
        return sparse.csr_matrix.__sub__(self, other)

    @staticmethod
    def getMatrixForTest():
        from cylp.py.utils.sparseUtil import csr_matrixPlus
        import numpy as np
        indptr = np.array([0, 2, 3, 6])
        indices = np.array([0, 2, 2, 0, 1, 2])
        data = np.array([1, 2, 3, 4, 5, 6])
        return csr_matrixPlus((data, indices, indptr), shape=(3, 3))


def sparseConcat(a, b, how, v_offset=0, h_offset=0):
    '''
    Concatenate two sparse matrices, ``a`` and ``b``, horizontally if
    ``how = 'h'``, and vertically if ``how = 'v'``.
    Add zero rows and columns if dimensions don't align.
    ``v_offset`` specifies how to align ``b`` along side ``a``. The
    value of ``v_offset`` will be added to each row index of ``b``.
    ``v_offset=-1`` means that we want the greatest possible offset without
    changeing the dimensions.
    ``h_offset`` is a similar argument but to specify horizontal offset.

    **Usage**

    >>> from scipy import sparse
    >>> from cylp.py.utils.sparseUtil import sparseConcat, csc_matrixPlus
    >>> s1 = csc_matrixPlus.getMatrixForTest()
    >>> s2 = sparse.lil_matrix([[1,0,2],[0,5,0]])
    >>> sparseConcat(s1, s2, 'v').todense()
    matrix([[1, 0, 4],
            [0, 0, 5],
            [2, 3, 6],
            [1, 0, 2],
            [0, 5, 0]])
    >>> sparseConcat(s1, s2, 'h').todense()
    matrix([[1, 0, 4, 1, 0, 2],
            [0, 0, 5, 0, 5, 0],
            [2, 3, 6, 0, 0, 0]])

    If ``a = None`` then return ``b``. This makes possible an incremental
    construction of large sparse matrices from scratch without the hassle
    of the initial value check.

    >>> s3 = None
    >>> ((sparseConcat(s3, s1, 'h').todense() == s1.todense()).all() and
    ...  (sparseConcat(s3, s1, 'v').todense() == s1.todense()).all())
    True

    '''
    if a is None:
        if b is None:
            return None
        return csr_matrixPlus(b)
    if b is None:
        return csr_matrixPlus(a)
    assert(h_offset >= -1 and v_offset >= -1)

    a = sparse.coo_matrix(a)
    b = sparse.coo_matrix(b)

    if how == 'h':

        if v_offset == -1:
            assert(a.shape[0] > b.shape[0])
            v_offset = a.shape[0] - b.shape[0]
        assert(h_offset >= 0)

        row = np.concatenate((a.row, b.row + v_offset), axis=0)
        col = np.concatenate((a.col, b.col + (a.shape[1] + h_offset)), axis=0)
        data = np.concatenate((a.data, b.data), axis=0)

        nRows = max(a.shape[0], b.shape[0] + v_offset)
        nCols = a.shape[1] + b.shape[1] + h_offset
        a = csr_matrixPlus((data, (row, col)),
                              shape=(nRows, nCols))

    elif how == 'v':
        if h_offset == -1:
            assert(a.shape[1] > b.shape[1])
            h_offset = a.shape[1] - b.shape[1]

        assert(v_offset >= 0)

        row = np.concatenate((a.row, b.row + (a.shape[0] + v_offset)), axis=0)
        col = np.concatenate((a.col, b.col + h_offset), axis=0)
        data = np.concatenate((a.data, b.data), axis=0)

        nCols = max(a.shape[1], b.shape[1] + h_offset)
        nRows = a.shape[0] + b.shape[0] + v_offset
        a = csr_matrixPlus((data, (row, col)),
                              shape=(nRows, nCols))
    return a


def I(n):
    '''
    Return a sparse identity matrix of size *n*
    '''
    if n <= 0:
        return None
    return csc_matrixPlus(sparse.eye(n, n))


